package BostonTumor;

import uchicago.src.sim.engine.Stepable;
import java.util.*; // Import ArrayList, Iterator
import java.awt.Dimension;

/**
 * Title:        Tumor Model Demo 2003
 * Description:  Demo version
 * Copyright:    Copyright (c) 2003
 * Company:      Massachusetts Institute of Technology + MGH
 * @author Yuri Mansury + Chaitanya Athale
 * @version 4.0
 * with ver 3.0 of compartment and network
 */

public class Agent
    implements Stepable {

  protected int x, y; // The agent's location on the grid
  protected int newX, newY; // The agent's new location on the grid
//  protected int[][] trackXY;//tracking the x,y coordinates
  TumorModel model; // Reference (handle) to the model
//  Compartment cellcompart; //Subcellular subcompartment
  protected int agentID; // The agent's identification
  protected int clusterID; // The ID of the cluster that an agent belongs to

//  boolean isOnClusterSurface; // Is agent on the surface of a cluster?
  boolean isProlif; // Is agent proliferating?
  boolean isProlifSet; // Is proliferation already decided?
  boolean isValueSet; // Is the value of this location already ulated?
  boolean isCoreAg; // Is this agent part of the spheroid core?
  boolean isEdgeAg; // Is this agent part of the spheroid edge?
  boolean isQuiesc; // Is agent quiesceing?
  boolean isMig; // Is agent migrating?
  boolean is3QFirst; //Is this cell the first to reach 3Q?

  // static variable to keep track how many tumor cells have already moved
  static int alreadyMoved;

  // Arraylist keeping track of the coordinates of the best places:
  ArrayList bestLoc;
  //ArrayList idealLoc;
  // NOTE: Initialization of EACH variable (16) in EACH of the 4 compartments
  double[][] Xmol_ia = {
      {
      0.0, 25.0, 0.0, 0.0,
      0.0, 0.0, 0.0, 0.1,
      0.1, 0.0, 1.0, 0.0,
      1.0, 5.0, 0.0, 0.0}
      , //compartment 0

      {
      0.0, 25.0, 0.0, 0.0,
      0.0, 0.0, 0.0, 0.1,
      0.1, 0.0, 1.0, 0.0,
      1.0, 5.0, 0.0, 0.0}
      , //compartment 1
      {
      0.0, 25.0, 0.0, 0.0,
      0.0, 0.0, 0.0, 0.1,
      0.1, 0.0, 1.0, 0.0,
      1.0, 5.0, 0.0, 0.0},
      //compartment 2
      {
      0.0, 25.0, 0.0, 0.0,
      0.0, 0.0, 0.0, 0.1,
      0.1, 0.0, 1.0, 0.0,
      1.0, 5.0, 0.0, 0.0} };

//compartment 3
  double[] rFlux ={0.003,//0.003,        // 0: LR complex formation rate of TGFaEGFR_s and activation (PP)
      0.0003, //3*10^(-4),             // 1: min^-1 2LR complex dissociation rate of 2xTGFaEGFR_s
      0.2, //2.2 * (10^( -1)),       // 2: Rate of LR internalization
      0.00005, // 3: Forward dissociation of TGFaEGFR_i: LR->L+R
      0.000233, // 4: Reverse dissoc. of TGFaEGFR_i : L+R->LR
      0.06, ///0.1,                  // 5: Rate of secretion of TGFa_i
      0.01, ///0.01                  // 6: Rate of EGFR_i insertion into the membrane
      0.1, ///0.01                // 7: Rate of EGFR_s internalization
      0.797, // 8: Basal transcription rate TGFa_rna
      0.144, // 9: Basal transcription rate EGFR_rna
      10.0, ///1.0,                    // 10: Km TGFa_rna transcriptional activation
      5.0, ///5.0,                    // 11: kind TGFa_rna transcriptional activation
      10.0, ///10.0,                   // 12: TGFa Hills coeff of induction of transcription
      1.00, ///1,                      // 13: TGFa weight of hills coeff
      10.0, //                    // 14: Km EGFR_rna transcriptional downregulation
      0.001, // 15: kind EGFR_rna transcriptional downregulation
      0.001, //10.0,                   // 16: EGFR Hills coeff of repression of transcription
      1.0, //1.0,                    // 17: EGFR weight of hills coeff
      0.332, ///0.5,                   // 18: [molecules/min] TGFa_rna rate of translation
      0.332, //0.5,                   // 19: [molecules/min] EGFR_rna rate of translation
      0.0012, // 20: TGFa_rna degradation rate
      0.0012, // 21: EGFR_rna degradation rate
      0.00167, // 22: TGFa_i degradation rate: TGFa_i --> a.a.
      0.00167, // 23: EGFR_i degradation rate: EGFR_i --> a.a.
      0.2, ///0.2                  // 24: Rate of LR phosphorylation
      0.01, //0.01                 // 25: Rate of pLR de-phosphorylation
      0.1, //0.1,                  // 26: kplca = knet(27); % [mol^-1 min^-1] Activation of PLC
      0.0001, //0.0002,                 // 27: kplci = knet(28); % [min^-1] In-Activation of PLC
      0.0, //0.5,                  // 28: kpkca % [mol^-1 min^-1] Activation of PKC-delta
      0.0, //0.0001,                 // 29: kpkci = knet(30); % [min^-1] In-Activation of PLC
      10.0, //5.0,                      // 30: Km PLC inhibition of LR*
      0.025, //0.25,                   // 31: kind PLC inhibition of LR*
      1.0, //5.0,                    // 32: Hills coeff PLC inhibition of LR*
      1.0, //1.0,                      // 33: weight of hills coeff PLC inhibition of LR*
      0.0, //1.0,                    // 34: Km of migratory potential
      0.0, //1.0,                    // 35: Vmax of migratory potential
      0.0, //2.0,                    // 36: Km of proliferative potential
      0.0, //1.0,                    // 37: Vmax of proliferative potential
      0.7, //0.1,                    // 38: Uptake of Glucose (Nutrients)
      5.0, //1,                      // 39: Weight of Glucose induced increased LR phosphorylation
      1, //10                   // 40: WEIGHT OF FOOD GRADIENT
      0.01};                   //41: Glucose metabolization rate




  double dtime = 0.1;
  /* 0: TGFa_ex
     1: EGFR_s: Schoeberl et al
     2: TGFaEGFR_s
     3: pLR_s (phos-TGFaEGFR_s)
     // Cytoplasmic species
     4: LR_i
     5: TGFa_i
     6: EGFR_i
     // nuclear
     7: TGFa_rna
     8: EGFR_rna
     // Feedback attentuation of Migratory signal
     9: PLCg
     10: PLCg_total
     11: PKC
     12: PKC_total
     13: Nucleotides
     14: Glucose_i
     15: External Glucose = Nutrients from Environment
   */
  //Arrays for the new values based on computation of the compartmental network
  double[][] Xmol_new = new double[4][16];
  //double[] Mig_new = new double[4];
  //double[] Prolif_new = new double[4];

  int [] phenoArr = {0, 0}; //Array that holds the last two states of the agent- [old, new]
  // 0 = no phenotype; 1= quiescent; 2= proliferative; 3= migratory
  double prolifLag;
  double migLag;
  double maxPLC;
  double maxGluci;
  double maxPLR;//max value of pp-TGFa-EGFR

  double maxGradPLC; //MAX change (in time of PLC conc)
  double[] c_gradPLC = new double[4]; //change in

  double[] gradPLC_i = new double[4]; //init PLCg-active concentration// 4 values for each compt
  double[] gradPLC_n =  new double[4]; //updated PLC conc


  double migWt = 1; //Weight of migration due to signalling network

  // Constructor
  public Agent(int i, TumorModel m) {
    agentID = i;
    clusterID = 0;
    model = m;
    prolifLag = 10;
    migLag = 0;
    isProlif = false;
    isProlifSet = false;
    isValueSet = false;
    isCoreAg = false;
    isEdgeAg = false;
    isQuiesc = false;
    is3QFirst = false;
    x = 0;
    newX = x;
    y = 0;
    newY = y;
    // Place agent initially at the origin
    model.agentTorus.putObjectAt(x, y, this);

    //initialize the track history
/*    trackXY[0][0]=1; //time pt
    trackXY[0][1]=this.x;
    trackXY[0][2]=this.y;
*/

   //Here we SET THE Rate of LR internalization.
    rFlux[2] = m.rLRint;
    //SET rate of Glucose internalization
    rFlux[38] = m.rFood;
    //SET rate of EGFR_s internalization to be identical to rLRint
    rFlux[7] = m.rLRint;
    //cellcompart.klri = m.rLRint;
    rFlux[11] = m.indTGFrna;



  }

  // Setting the agent's position on the grid
  public void placeTo(int a, int b) {
    if ((a != x) || (b != y)) {
      //not the same old location
      // First, remove the agent from the current location
      model.agentTorus.removeObjectAt(x, y, this);
      // Second, update the coordinates to those of the new location
      x = a;
      y = b;
      // Third, move the agent to the new location
      model.agentTorus.putObjectAt(x, y, this);
    }
    else
      x = x;
      y = y;
  }

  // Resetting the flags and emptying the arraylist
  public void reset() {
    isValueSet = false;

    bestLoc = null;


    isCoreAg = false;
    isEdgeAg = false;

    isQuiesc = false; //reset isQuiesc
    isMig = false;
    isProlif = false; // reset the boolean isProlif
    isProlifSet = false; // reset the boolean isProlifSet
    alreadyMoved = 0;

  }

  // Implementing the step method for the interface Stepable
  public void step() {

    //Invoke the network to calulcate values
    int molit = 10;//60=seconds/min; avg speed of cellmig=1um/20min; gridsize=10um


//    System.out.println("@Agent: EGFR surf: "+ Xmol_ia[1][1]
//                             + " TGFex: " +  Xmol_ia[1][0]);
    for (int moltime = 0; moltime < molit; moltime++) { //in order to iterate molecular network

      //more often than the cellular processes (exact factor to be determined)
      computeNetwork();
      compMaxGradPLC();
      updateMolecules();
      //makes the old phenotype the new one
      updatePheno();
    }

    if (!isQuiesc && !isProlif && isMig) {
     // System.out.println("Cell migrates");
      randomMigration(); // Agent migrates
    }
  }

  // The method that randomizes the selection of the location to migrate
  protected void randomMigration() {
     // Increment the number of already-moved agents

    int xLo = model.xLowerBound[x], xHi = model.xUpperBound[x];
    int yLo = model.yLowerBound[y], yHi = model.yUpperBound[y];
    if (!this.isValueSet) {
      bestLoc = new ArrayList();
      // Let the agent choose the best location to migrate to
      double value = 0.0; // Storing the value of a location
      double valuegl = 0.0;

      int compt = 0, mm = 0, pp = 0;
      double tempmig = 0.0; //Parameter that contributes to
      // migratory attractiveness of a locID
      int nFreeNb = model.noteEmptyNeighbor(x,y);//number of free nb locations

      if (nFreeNb > 0) {
        alreadyMoved++;
        migLag = 0;
        for (int i = xLo; i < xHi; i++) {
          int ii = model.agentTorus.xnorm(i);
          for (int j = yLo; j < yHi; j++) {
            int jj = model.agentTorus.ynorm(j);
            if (mm == 0 | mm == 2 | mm == 4 | mm == 6 | mm == 8) {
              mm++;
              continue;
            }
            else {
              tempmig = this.Xmol_new[compt][9]; //Mig_new[compt]; //attributing the migratory pot to a factor
              //Migration depends on PLC peak value
              value = tempmig; //(tempmig * migWt);
              valuegl = this.Xmol_new[compt][14];
              /*
                   NOTE: Migratory potential is added to the "value" of a LocID
                         First, determine the best location in the neighborhood
               */
              // errPLC is the fraction of the maximum that makes a location
              // good enough to migrate into
              // CHANGED# if (value == max) {
              //if (value >= this.maxPLR){
              //if (valuegl >= this.maxGluci- this.maxGluci*model.errPLC){
              // - (this.maxGluci*0.1)){
              //if (value >= model.threshPLC){
              //if (value == this.maxGradPLC){// && value >= model.threshPLC){
              // if (value == this.maxPLC){
              //if (value == this.maxPLC){
              if (value >= (this.maxPLC- this.maxPLC*model.errPLC)) {
              //  System.out.println("Error PLC= " + model.errPLC);
                if (model.agentTorus.getCellSizeAt(ii, jj) == 0) {
                  bestLoc.add(new Dimension(ii, jj));
                }
              }
              compt++;
            }
            mm++;
          }
        }
      } //if the neighbours are free
      /*else if(nFreeNb == 0){
        isMig = false;
        isQuiesc = true;
        isProlif = false;
      }*/
      else{
      //System.out.println("cant move");
      }
       if(bestLoc.size()==0){
        bestLoc.add(new Dimension(x,y));//if the bestLoc has no elements-old coords
      }
    }

    //HERE: by making the precision=model.searchPrecision we remove any randomness in
    //bestLoc choice; However we can control this by making it greater than precision
    //and therefore completely random, as a test for the effect of totally random motion
    double precision =model.searchPrecision;//CHANGE PRECISION MAXXX! model.getNextDoubleFromTo(0.0, 1.0);

    if (precision <= model.searchPrecision) { // If random number is less than search precision
      chooseBestLoc(); // then choose best location available
    }
    else if (precision > model.searchPrecision) { // Else, pick a random location
      Dimension xyRandom = model.getEmptyNeighbor(x, y);
      int randomNb = model.noteEmptyNeighbor(x,y);
      //int xRandom = xyRandom[0];
      //int yRandom = xyRandom[1];
      if (randomNb > 0){
           //model.agentTorus.getCellSizeAt(xRandom, yRandom) == 0) {
        // If random location is not current
        //and the random empty neighborhood is non-zero (i.e. no empty neighborhood)
        newX = xyRandom.width;
        newY = xyRandom.height; // Update the new-location coordinates
        model.movingList.add(this); // Add this agent to the moving list
      }
      else
        newX = x;//dont move if no free neighbour
        newY = y;//and noteEmptyNeighbor turns up no non-self free grid pt

    }
    // Now we check if the center of Quadrant 3 has been invaded
    if (!model.isThirdQuadrantReached) {
      flagThirdQuadrant();
      //System.out.println("Has the 3rd Quadrant been reached: Random Migration?");
    }

  }


public void initExtMol(){
    int xLo = model.xLowerBound[x], xHi = model.xUpperBound[x];
    int yLo = model.yLowerBound[y], yHi = model.yUpperBound[y];
    int locID =0, compn = 0, cono = 0;

    for (int i = xLo; i < xHi; i++) {
      int ii = model.agentTorus.xnorm(i);
      for (int j = yLo; j < yHi; j++) {
        int jj = model.agentTorus.ynorm(j);

        locID = ii * model.resolution + jj;
        if(cono ==0||cono ==2||cono ==4||cono ==6||cono ==8){
          cono++;
          continue;}
        else{
            Xmol_ia[compn][15] = model.oldfoodLevel[locID];
            Xmol_ia[compn][0] = model.oldAutocrFood[locID];

          compn++;
          cono++;
        }
      }
    }
  }

////////////////////////////////////////////////////////////////////////////////////
  /* Compartment-Network module: Calculate molecular values for subcomp when called
   */
  protected void computeNetwork() {
     //System.out.println("OLD LR= "+ Xmol_ia[0][2]);
    initExtMol(); //initialize external molecules

    int locID; //Location ID
    int mm = 0,
        compID = 0, //the compartment ID (0-3)
        compIDp1 = 1, //compart. ahead by 1
        compIDm1 = 3; //compart. behind by 1
    int xLo = model.xLowerBound[x], xHi = model.xUpperBound[x];
    int yLo = model.yLowerBound[y], yHi = model.yUpperBound[y];
    int locIDag;//locan ID on which the agent sits
    locIDag = x * model.resolution + y;
    //System.out.println("update network food");
    //System.out.println("Initial EGFRs "+Xmol_ia[2][1]);
    double Mass = 0.0;
    for (int i = xLo; i < xHi; i++) {
      int ii = model.agentTorus.xnorm(i);
      for (int j = yLo; j < yHi; j++) {
        int jj = model.agentTorus.ynorm(j);
        locID = ii * model.resolution + jj;
        ///////

        if (mm == 0 | mm == 2 | mm == 4 | mm == 6 | mm == 8) {
          mm++;
          continue;
        }
        else {

          removeNegval(compID);
           //The compartment ID number can be only between 0 and 3
          if (compID == 0) {
            compIDm1 = 3;
          }
          else if (compID == 3) {
            compIDp1 = 0;
          }
          else {
            compIDp1 = compID + 1;
            compIDm1 = compID - 1;
          }

//Compartments created using the simpler constructor that ASSUMES
          //total compartments=4; connectivity=2
          Compartment c1 = new Compartment(compID, Xmol_ia[compID],rFlux);
          Compartment cp1 = new Compartment(compIDp1, Xmol_ia[compIDp1],rFlux);
          Compartment cm1 = new Compartment(compIDm1, Xmol_ia[compIDm1],rFlux);
//COmputing the new Values of the molecules in each compartment of the network


          Xmol_new[compID][0] = Xmol_ia[compID][0] +
              dtime * c1.computeTGFa_ex_c() ;

          //EGFR_s
          Xmol_new[compID][1] = Xmol_ia[compID][1] +
              dtime * c1.computeEGFRs_c(cp1, cm1);

          //TGFaEGFR_s
          Xmol_new[compID][2] = Xmol_ia[compID][2] +
              dtime * c1.computeTGFaEGFRs_c(cp1, cm1);

          //pLR_s
          Xmol_new[compID][3] = Xmol_ia[compID][3] +
              dtime * c1.computepLR_sc(cp1, cm1);

          //TGFaEGFR_i
          Xmol_new[compID][4] = Xmol_ia[compID][4] +
              dtime * c1.computeTGFaEGFRi_c(cp1, cm1);

          //TGFa_i
          Xmol_new[compID][5] = Xmol_ia[compID][5] +
              dtime * c1.computeTGFai_c(cp1, cm1);

          //EGFR_i
          Xmol_new[compID][6] = Xmol_ia[compID][6] +
              dtime * c1.computeEGFRi_c(cp1, cm1);

          //TGF-alpha transcript
          Xmol_new[compID][7] = Xmol_ia[compID][7] +
              dtime * c1.computeTGFrna_c(cp1, cm1);

          //EGFR transcript
          Xmol_new[compID][8] = Xmol_ia[compID][8] +
              dtime * c1.computeEGFRrna_c(cp1, cm1);


          gradPLC_i[compID] =  Xmol_ia[compID][9];// initial PLC conc
          //System.out.println("OLD: "+gradPLC_i[compID]);
          //PLC active
          Xmol_new[compID][9] = Xmol_ia[compID][9] +
              dtime * c1.computePLC_c(cp1, cm1);

          gradPLC_n[compID] = Xmol_new[compID][9];//New PLC conc
          //System.out.println("NEW: "+gradPLC_n[compID]);//


          //PLC total
          Xmol_new[compID][10] = Xmol_ia[compID][10]; //+ dtime *c1.computePLCi_c(cp1, cm1);

          //PKC Active
          Xmol_new[compID][11] = Xmol_ia[compID][11] +
              dtime * c1.computePKC_c(cp1, cm1);

          //PKC inactive
          Xmol_new[compID][12] = Xmol_ia[compID][12]; // + dtime *c1.computePKCi_c(cp1, cm1);

          // Nucleotides
          Xmol_new[compID][13] = Xmol_ia[compID][13] +
              dtime * c1.computeNts(cp1, cm1);

          // Amino Acids Internal
          Xmol_new[compID][14] = Xmol_ia[compID][14] +
              dtime * c1.computeGluci_c(cp1, cm1);

          Xmol_new[compID][15] = Xmol_ia[compID][15]+
              dtime * c1.computeGluc_exc();
        //  Mig_new[compID] = c1.computeMig_c(Xmol_new[compID][9]);
//MIGRATION IMPULSE = PLCg active molecules

          //Only for those 4 neighbors in the proxim. of the 4 compartments
          //accumulating the values of Autocrine Secretion and
          model.autocrFood[locID] = Xmol_new[compID][0];//CHAI: addtive OR NOT??
          model.foodLevel[locID] = Xmol_new[compID][15];//represents depletion of food

          for (int molecn = 0; molecn < 14; molecn++) {
            Mass += Xmol_new[compID][molecn];
          }

//testing our model hypoth-> prolif, then gluc->Mig
       //  if(Xmol_new[compID][14] > 0){
       //   System.out.println("wave reaches AND PLCg= "+ Xmol_new[compID][9]);
       // }
           //if(Xmol_new[compID][9] > model.threshPLC){
           //System.out.println("comp "+compID+" Gluci"+Xmol_new[compID][14]);//"PLCg= " + Xmol_new[compID][9] +
         //                     ", ppLR= " +Xmol_new[compID][3]);
           //}
        // }
          removeNegval(compID);
          compID++; //iterator for compartment
          mm++;
        }
      }
    }

    computeMaxPLC();//computes the maximal value of PLCg
   //compute the maximal change in time of PLCg: Dittmar et al 2002


//model.oldAutocrFood = model.autocrFood;
    //model.oldfoodLevel = model.foodLevel;
    //System.out.println("network updates old food");
    //Migratory impulse

//gradPLC[compID] = Xmol_new[compID][9] - Xmol_ia[compID][9];



//    updateMolecules(); //Updates initial to new
  }

  //EOF computeNetwork
  //This method updates the arrays holding molecular concentrations
  public void updateMolecules() {
//      public void updateMolecules(int compno) {
//    Xmol_ia[compno] = Xmol_new[compno];

    for (int compn = 0; compn < 4; compn++){
      this.Xmol_ia[compn] = this.Xmol_new[compn];
    }
  }

  //updates phenotype history to [old, new]
  public void updatePheno() {
    this.phenoArr[0] = this.phenoArr[1];
  }

/*
  //This method updates the arrays holding molecular concentrations
  public void updateMolecules(int compno) {
    Xmol_ia[compno] = Xmol_new[compno];
    //for (int moln = 0; moln < 16; moln++){
    // Xmol_ia[compno][moln] = Xmol_new[compno][moln];
    //}
  }
*/
  public void removeNegval(int compno) {
    int moln = 0;
    for (moln = 0; moln < 16; moln++) {

      if(Xmol_ia[compno][moln] > 10000){
         System.out.println("OUCH!  OUT OF RANGE!! molc#="+moln);
       }


      if (Xmol_new[compno][moln] < 0) {
        Xmol_new[compno][moln] = 0.0;
      }
      else
        if (Xmol_new[compno][moln] == 0) {
      }
    }
  }

  public void computeMaxPLC(){
    maxPLC = 0.0;
    maxGluci = 0.0;
    maxPLR  = 0.0;
    for(int cnum = 0; cnum<4;cnum++){
      //if a given compartment has PLCg higher than maxPLC make that the max
      maxPLC = (maxPLC < this.Xmol_new[cnum][9])? this.Xmol_new[cnum][9]:maxPLC;
      maxGluci = (maxGluci < this.Xmol_new[cnum][14])? this.Xmol_new[cnum][14]:maxGluci;
      maxPLR = (maxPLR < this.Xmol_new[cnum][3])? this.Xmol_new[cnum][3]:maxPLR;
    }
    //System.out.println("MAX PLCg= " +maxPLC);
  }



  public void compMaxGradPLC(){
    maxGradPLC = 0.0;
    double[] c_gradPLC = new double[4];
    //maxGradGluci = 0.0;
    //maxGradPLR  = 0.0;
    for(int cnum = 0; cnum<4;cnum++){
      c_gradPLC[cnum] = Math.abs(gradPLC_n[cnum] - gradPLC_i[cnum]);//new - old
      //if a given compartment has PLCg higher than maxPLC make that the max
      maxGradPLC = (maxGradPLC > c_gradPLC[cnum])? maxGradPLC: c_gradPLC[cnum];
    //        if(Xmol_new[cnum][14]>0){
    //         System.out.println("PLC gradient " + c_gradPLC[cnum]);
    //                            +" NEW "+gradPLC_n[cnum] );
    //         }
      }
    //      if(Xmol_new[3][14]>0){
    //    System.out.println("Gluc wave hits!");
    //    System.out.println("pLR in polariz dir: " + Xmol_new[3][3]);
    //System.out.println("max PLC grad-ag: " + maxGradPLC);}
  }



  //ENDOF compartment-network

//calculate the L-infinity distance from the 3rd quadrant centre
  public int dist3Quad(){
    int x3q =Math.abs(model.resolution * 3 / 4 - this.x),
              y3q = Math.abs(model.resolution / 4 - this.y);
    int d3Quad = model.computeDistance(x3q,y3q);
    return d3Quad;
  }


//calculate the L-infinity distance from the 3rd quadrant centre
  public int distEucl3Quad(){
    int[] xySource = {model.resolution * 3 / 4, model.resolution / 4};
    int[] agPos = {this.x,this.y};
    int d3EucQuad = model.computeEuclD(xySource, agPos);
    return d3EucQuad;
  }

  // Method that flag the center of Quadrant 3
  // If not, then we continue to monitor until it has been reached
  protected void flagThirdQuadrant() {
    //int dist3q = dist3Quad();//Linfinity distance
    int dist3q = distEucl3Quad();//Euclidean distance

    if(dist3q <= model.foodPeakRange){//if the agent is at the edge of the food peak
    //if ( (x == model.resolution * 3 / 4) && (y == model.resolution * 1 / 4)) {
    //OLD METHOD: if the agent is at the centre of the food peak
    //  dist3q= 0;
      model.isThirdQuadrantReached = true;
      model.timeToThirdQuadrant = (long) model.getTickCount();
      this.is3QFirst = true;
      int[] firstXY = {this.x, this.y};
      System.out.println(
          "At migrate(), we have invaded the center of Quadrant III!!!");
    }
  }

  // Method that selects the best location and move the agent there
  // if there are more than one locations with the same "goodness"
  // then the random number picks the better of the locations
  protected void chooseBestLoc() {
    int rndChoice = model.getNextIntFromTo(0, bestLoc.size() - 1);
    //System.out.println("no bestLocs "+bestLoc.size());
    Dimension loc = (Dimension) bestLoc.get(rndChoice);
    //HERE: there is only 1 IDEAL loc. REDUCES randomness
    //Dimension loc = (Dimension) idealLoc.get(0);
    //int nFreeNb = model.noteEmptyNeighbor(this.x,this.y);
    //if(nFreeNb > 0 && !(loc.width == x && loc.height == y)) { // If best location is not current
    if(!(loc.width == x && loc.height == y)) { // If best location is not current
    //if the cell has some free neighbours
        newX = loc.width;
        newY = loc.height; // Update the new-location coordinates
        model.movingList.add(this); // Add this agent to the moving list
      }
      else
        loc.width = x;//keep the old values
        loc.height = y;//and dont move

  }

/*  public void updateTrack(int timeAg){
    this.trackXY[timeAg][0]=timeAg;
    this.trackXY[timeAg][1]=this.x;
    this.trackXY[timeAg][2]=this.y;
  }*/

  // Method insuring replication of results is possible holding rngSeed fixed
  public int hashCode() {
    return agentID;
  }

}
