package BostonTumor;

/**
 * <p>Title: compartment class </p>
 * <p>Description: This implements a cellular sub-compartmentation</p>
 * <p>Copyright: Chaitanya A. Athale (c) 2003</p>
 * <p>Company: Mass. General Hospital</p>
 * @author Chaitanya Athale
 * @version 1.0
 */
//imports
//class definition
public class Compartment
    extends Network {
  /* Define components
  kin = influx rate constant
  This value as well as
  kout are dependent on biophysical properties of the the diffusing molecue,
  and the nature of the underlying medi
um/matrix. Therefore these values are necessarily different for
 the various molecules

Typical Diffusion constants of transcription factors in the nucleus:  0.3 um^2/s
Lenght of "square" cell: 10 um, Area: 100 um^2, Compartmental size: 25 um^2
Time to cover one compartment: 0.3/25 = 0.012 /s
*/
  static double[] kin = {0.01,// 0: EGFR_s
      0.001,  // 1: TGFaEGFR_s
      0.00, // 2: pLR_s
      0.001,   // 3: LR_i
      1/10^(5),   // 4: L_i
      1/10^(5),   // 5: R_i
      1/10^(5),   // 6: L_rna
      1/10^(5),   // 7: R_rna
      0.0, // 8: plc
      //0.005,   // : plci
      0.0, //[s^-1] 9: pkc
      //0.001, // : pkci
      0.67, // 10: glucose_internal
      0.7}, // 11: nts

      kout =  {0.01,// 0: EGFR_s
      0.001,  // 1: TGFaEGFR_s
      0.00, // 2: pLR_s
      0.001,   // 3: LR_i
      1/10^(5),   // 4: L_i
      1/10^(5),   // 5: R_i
      1/10^(5),   // 6: L_rna
      1/10^(5),   // 7: R_rna
      0.0, // 8: plc
      //0.005,   // : plci
      0.0, //[s^-1] 9: pkc
      //0.001, // : pkci
      0.67, // 10: glucose_internal
      0.7}; // 11: nts



//change to proteced type
  //call in tumor model, so they dont get repeatedly created and destroyed

  int connect, //how many comparments connected to any given one
      comp_ID, //unique identifier of the compartment under consideration
      total_neigh; //total number of comparments in a single cell

/* NOTE:
   We use multiple (here: 2) constructors for setting up Compartment, since
 certain typical situations, like a connectivity of 2 mean a reduced number of
 inputs that would neet to be passed
  */

  //The constructor of the subclass compartment with flux parameters
  public Compartment(int connect, int comp_ID, int total_neigh,
                     double[] old_molX, double[] kRates) {
    super(old_molX, kRates);
    this.kin = kin;
    this.kout = kout;
    this.connect = connect;
    this.comp_ID = comp_ID;
    this.total_neigh = total_neigh;
   // this.knet = kRates;
  }

//this constructor calls a compartment with a compartment position and concentration of
//the molecular components X[0] to X[n] for n reactants
  public Compartment(int comp_pos, double[] old_molX, double[] kRates) {
    this(2, comp_pos, 4, old_molX, kRates);
  }

  /*  public Compartment(int comp_pos){
    this(2, comp_pos, 4, 0.0, 0.0, 0.0, 0.0);
   }
   */

//TGFa_ex_c Compartmental - simple extension of the network eqn
  public double computeTGFa_ex_c() {
    double n_cTGFe = computeTGFa_ex();
    return n_cTGFe;
  }

//EGFR_s
// Calculates compartmental mass of EGFR_s in a compartment
// based on as well its old values and those of the neighboring compartments
  public double computeEGFRs_c(Compartment cp1, Compartment cm1) {
    //the agent invokes the 3 compartments at a given moment
    //the values of the neighboring +1 (cp1) and -1(cm1) compartments are
    //then passed on to the method
    double n_cEGFRs = this.computeEGFR_s()
        - this.EGFR_s*kout[0]*2
        + kin[0] * cp1.EGFR_s + kin[0] * cm1.EGFR_s;

    return n_cEGFRs;
  }

// TGFaEGFR_s surface complex (dimeric, phosphorlylated, active)
  public double computeTGFaEGFRs_c(Compartment cp1, Compartment cm1) {
    double n_ctgfaEGFRs = this.computeTGFaEGFR_s() - this.TGFaEGFR_s*kout[1]*2
        + kin[1] * cp1.TGFaEGFR_s
        + kin[1] * cm1.TGFaEGFR_s;
    return n_ctgfaEGFRs;
  }

  // pLR_s surface complex (dimeric, phosphorlylated, active)
  public double computepLR_sc(Compartment cp1, Compartment cm1) {
    double n_pLRs = this.computepLR_s() - (this.pLR_s* kout[2] * 2)
        + kin[2] * cp1.pLR_s
        + kin[2] * cm1.pLR_s;
    return n_pLRs;
  }


// The internalized TGF-alpha EGFR complex (dimeric)
  public double computeTGFaEGFRi_c(Compartment cp1, Compartment cm1) {
    double n_cLRi = this.computeTGFaEGFR_i()
        - (this.LR_i * kout[3] * 2)
        + (kin[3] * cp1.LR_i)
        + (kin[3] * cm1.LR_i);
    return n_cLRi;
  }

// TGF-alpha cytoplasmic protein compartmental
  public double computeTGFai_c(Compartment cp1, Compartment cm1) {
    double n_cLi = this.computeTGFa_i() - this.TGFa_i * kout[4] * 2
        + kin[4] * cp1.TGFa_i
        + kin[4] * cm1.TGFa_i;
    return n_cLi;
  }

// EGFR cytoplasmic protein compartmental
  public double computeEGFRi_c(Compartment cp1, Compartment cm1) {
    double n_cRi = this.computeEGFR_i()
        - this.EGFR_i*kout[5]*2
        + kin[5] * cp1.EGFR_i
        + kin[5] * cm1.EGFR_i;
    return n_cRi;
  }

  // TGF-alpha RNA nuclear compartmental
  public double computeTGFrna_c(Compartment cp1, Compartment cm1) {
    double n_cTGFrna = this.computeTGFa_rna()
        - this.TGFa_rna*kout[6]*2
        + kin[6] * cp1.TGFa_rna
        + kin[6] * cm1.TGFa_rna;
    return n_cTGFrna;
  }

  // EGFR RNA nuclear compartmental
  public double computeEGFRrna_c(Compartment cp1, Compartment cm1) {
    double n_cEGFRrna = this.computeEGFR_rna()
        - this.EGFR_rna * kout[7]*2
        + kin[7] * cp1.EGFR_rna
        + kin[7] * cm1.EGFR_rna;
    return n_cEGFRrna;
  }

// PLCgamma active compartmental
public double computePLC_c(Compartment cp1, Compartment cm1) {
  double n_cPLC = this.computePLC()
      - this.PLC * kout[8] * 2
      + kin[8] * cp1.PLC
      + kin[8] * cm1.PLC;
  return n_cPLC;
}

// PLCgamma TOTAL compartmental change
public double computePLCt_c() {
  double n_cPLCt = this.computePLCt();
   //   - this.PLCi * kout[9] * 2
   //   + kin[9] * cp1.PLCi
   //   + kin[9] * cm1.PLCi;
  return n_cPLCt;
}

// PKC delta active compartmental
public double computePKC_c(Compartment cp1, Compartment cm1) {
  double n_cPKC = this.computePKC()
      - this.PKC * kout[9] * 2
      + kin[9] * cp1.PKC
      + kin[9] * cm1.PKC;
  return n_cPKC;
}

// PKCi delta INACTIVE compartmental
public double computePKCt_c() {
  double n_cPKCt = this.computePKCt();
    //  - this.PLCi * kout[11] * 2
    //  + kin[11] * cp1.PKCi
    //  + kin[11] * cm1.PKCi;
  return n_cPKCt;
}


  // Amino Acids inside the compartment
public double computeGluci_c(Compartment cp1, Compartment cm1) {
  double n_glucic = this.computeGluci()
       - this.Gluc*2*kout[10]
       + kin[10] * cp1.Gluc
        + kin[10] * cm1.Gluc;
  return n_glucic;
}

// Amino Acids OUTside the compartment
public double computeGluc_exc() {
double n_aa_ex = computeGluc_ex();
//GRADIENT OF AmAc_ex to AmAc internal FOR THAT COMPARTMENT
return n_aa_ex;
}

// Nucleotides in the nucleus
public double computeNts(Compartment cp1, Compartment cm1) {
  double n_Nts = this.computeNts() - (this.Nts*2*kout[11])
      + (kin[11]*cp1.Nts) + (kin[11]*cm1.Nts);
  return n_Nts;
}

  public double computeMig_c(double c_PLC_new) {
//the migratory impulse is called from network class as a factor of
    //EGFR-TGF-alpha complex AND internal AMINO ACID stores
    double n_Mig = migimpulse(c_PLC_new);
    return n_Mig;
  }

  public double computeProlif_c(double c_pLRs_new) {
//the migratory impulse is called from network class as a factor of
    //EGFR-TGF-alpha complex AND internal AMINO ACID stores
    double n_Prol = prolifpot(c_pLRs_new);
    return n_Prol;
  }


}
