package BostonTumor;

/**
 * <p>Title: network.java</p>
 * <p>Description: The arithemetic solution to the TGF-alpha -EGFR
 * network that takes TGF-alpha concentration of a cell and produces a
 * migratory output</p>
 * <p>Copyright: Chaitanya A. Athale (c) 2002</p>
 * <p>Company: MGH/CBML</p>
 * @Chaitaya A. Athale
 * @version 1.0
 */

public class Network {
  // Define components
  double
 // cell surface species
      TGFa_ex,// 0: TGFa_ex
      EGFR_s, // 1: EGFR_s: Schoeberl et al
      TGFaEGFR_s,// 2: TGFaEGFR_s
      pLR_s, //3: pLR_s (phos-TGFaEGFR_s)
// Cytoplasmic species
      LR_i,//4: LR_i
      TGFa_i,//5: TGFa_i
      EGFR_i,//6: EGFR_i
// nuclear
      TGFa_rna,//7: TGFa_rna
      EGFR_rna, //8: EGFR_rna
// Feedback attentuation of Migratory signal
      PLC, // 9: PLCg
      PLCt,//  10: PLCg_total
      PKC, // 11: PKC
      PKCt,// 12: PKC_total
      Nts, // 13: Nucleotides
      Gluc, // 14: Glucose taken up by the cell
      Gluc_ex; // 15: External Glucose = Nutrients that arrive from external source

  //Rate constants of the network reactions/
  //NOTE: the "static" term fixes the values and makes them a class variable
  //they can be accessed as "Network.k12" for example.
//define in tumor model, give a tumor model handle with these constants and
  //values of varialbles etc.
/*  protected double[] //reaction parameters with fixed values
      knet = {0.2,//0.003,        // 0: LR complex formation rate of TGFaEGFR_s and activation (PP)
      0.1,//3*10^(-4),             // 1: min^-1 2LR complex dissociation rate of 2xTGFaEGFR_s
      0.001,//2.2 * (10^( -1)),       // 2: Rate of LR internalization
      0.1,                   // 3: Forward dissociation of TGFaEGFR_i: LR->L+R
      0.02,                // 4: Reverse dissoc. of TGFaEGFR_i : L+R->LR
      0.0001,///0.1,                  // 5: Rate of secretion of TGFa_i
      0.01,///0.01                  // 6: Rate of EGFR_i insertion into the membrane
      0.001,///0.01                // 7: Rate of EGFR_s internalization
      0.0,                   // 8: Basal transcription rate TGFa_rna
      0.0,                   // 9: Basal transcription rate EGFR_rna
      1.0,///1.0,                    // 10: Km TGFa_rna transcriptional activation
      5.0,///5.0,                    // 11: kind TGFa_rna transcriptional activation
      10.0,///10.0,                   // 12: TGFa Hills coeff of induction of transcription
      1.0,///1,                      // 13: TGFa weight of hills coeff
      0.0,//                    // 14: Km EGFR_rna transcriptional activation
      0.0,                    // 15: kind EGFR_rna transcriptional activation
      0.0,//10.0,                   // 16: EGFR Hills coeff of induction of transcription
      0.0,//1.0,                    // 17: EGFR weight of hills coeff
      0.0,///0.5,                   // 18: [molecules/min] TGFa_rna rate of translation
      0.0,//0.5,                   // 19: [molecules/min] EGFR_rna rate of translation
      0.0,                  // 20: TGFa_rna degradation rate
      0.0,                  // 21: EGFR_rna degradation rate
      0.0,                // 22: TGFa_i degradation rate: TGFa_i --> a.a.
      0.0,                // 23: EGFR_i degradation rate: EGFR_i --> a.a.
      0.02,///0.2                  // 24: Rate of LR phosphorylation
      0.01,//0.01                 // 25: Rate of pLR de-phosphorylation
      0.10,//0.1,                  // 26: kplca = knet(27); % [mol^-1 min^-1] Activation of PLC
      0.0016,//0.0002,                 // 27: kplci = knet(28); % [min^-1] In-Activation of PLC
      0.10,//0.5,                  // 28: kpkca % [mol^-1 min^-1] Activation of PKC-delta
      0.01,//0.0001,                 // 29: kpkci = knet(30); % [min^-1] In-Activation of PLC
      5.0,//5.0,                      // 30: Km PKC inhibition of LR*
      0.25,//0.25,                   // 31: kind PKC inhibition of LR*
      5.0,//5.0,                    // 32: Hills coeff PKC inhibition of LR*
      1.0,//1.0,                      // 33: weight of hills coeff PKC inhibition of LR*
      0.0,//1.0,                    // 34: Km of migratory potential
      0.0,//1.0,                    // 35: Vmax of migratory potential
      0.0,//2.0,                    // 36: Km of proliferative potential
      0.0,//1.0,                    // 37: Vmax of proliferative potential
      0.1,//0.1,                    // 38: Uptake of Glucose (Nutrients)
      0.0,//1,                      // 39: Weight of effect of internal Glucose on migratory and prolif pot
      0.0}; //10                   // 40: WEIGHT OF FOOD GRADIENT
*/
//double[] knet = new double[41];
      /* CONSTANTS */
     double
      kf,// = knet[0],       // LR complex formation rate of TGFaEGFR_s and activation (PP) Ls+Rs->LRs
      kr,// = knet[1],       // LR complex dissociation rate of 2xTGFaEGFR_s LRs->Ls+Rs
      klri,// = knet[2],     // LR internalization rate
      kif,// = knet[3],      // forward reaction of LR_i dissociation LRi->Li+Ri
      kir,// = knet[4],      // reverse reaction of LR_i dissociation, Li+Ri->LRi
      kls,// = knet[5],      // rate of secretion of TGFa_i
      krs,// = knet[6],      // rate of secretion of EGFR_i
      kri,// = knet[7],      // rate of internalization of EGFR_i

      ktil,// = knet[8],    // TGFa_rna Basal transcription rate
      ktir,// = knet[9],   // EGFR_rna Basal transcription rate

      Km1,// = knet[10],    // Km TGFa_rna transcriptional activation
      kind1,// = knet[11],  // kind TGFa_rna transcriptional activation
      khil1,// = knet[12],  // TGFa Hills coeff of induction of transcription
      kwh1,// = knet[13],   // weight of hills coeff

      Km2,//= knet[14],    // Km EGFR_rna transcriptional activation
      kind2,//= knet[15],  // kind EGFR_rna transcriptional activation
      khil2,//= knet[16],  // Hills coeff of induction of transcription
      kwh2,//= knet[17],   // weight of hills coeff

      ktral,// = knet[18],  // TGFa_rna rate of translation
      ktrar,// = knet[19],  // EGFR_rna rate of translation
      krdl,// = knet[20],   // TGFa_rna rate of degradation
      krdr,// = knet[21],   // EGFR_rna rate of degradation
      kpdl,// = knet[22],   // TGFa_i rate of degradation: TGFa_i --> a.a.
      kpdr,// = knet[23],   // EGFR_i rate of degradation: EGFR_i --> a.a.

      kpho,// = knet[24],   // min^-1 Phosphorylation of LR_s
      kdpho,// = knet[25],  // min^-1 Dephosphorylation of pLR_s

      kplca,// = knet[26], // [mol^-1 min^-1] Activation of PLC
      kplci,// = knet[27], // [min^-1] In-Activation of PLC
      kpkca,// = knet[28], // [mol^-1 min^-1] Activation of PKC-delta
      kpkci,// = knet[29], // [min^-1] In-Activation of PLC

      Km3,// = knet[30],      // [M] Km PLC inhibition of pLR*
      kfeed1,// = knet[31],   // [min^-1] kind PLC inhibition of pLR*
      kfeedh,// = knet[32],   // Hills coeff PLC inhibition of pLR*
      kfeedhw,// = knet[33],  // weight of hills coeff PLC inhibition of pLR*

      Km4,// = knet[34],      // Km of migratory potential
      Vm4,// = knet[35],      // Vmax of migratory potential

      Km5,// = knet[36],      // Km of Proliferative potential
      Vm5,// = knet[37],      // Vmax of Proliferative potential

      kglucin,// = knet[38],   // Intake of Glucose from environment
      gluc_wt ,//= knet[39],   // Weight of Glucose influence on migration and proliferation
      wfdgrad,//= knet[40];// Weight of Glucose gradient
     kglucMet;//knet[41]       Rate of glucose metabolization


  /* Global constants
       */
//  double Nav = 6.022 *(10^23), // Avogadros number of molecules in 1 gram mole
 // vol_c = (4.2/4)*(10^(-12)), // Litres volume of nucleus
  //  vol_n = (5.2/4)*(10^(-13)); // Litres volume of cell

 // Build constructor
  //the network of reactions has the following properties
  //thus it can be initialized everytime it is called with these values
  public Network(double[] old_molX, double[] kRates) { // Network must be given the old values of all
    //molecules to be calculated, 15 at current count

    // cell surface species
    this.TGFa_ex = old_molX[0];     // 0: TGFa_ex
    this.EGFR_s= old_molX[1];       // 1: EGFR_s: Schoeberl et al
    this.TGFaEGFR_s = old_molX[2];  // 2: TGFaEGFR_s
    this.pLR_s= old_molX[3];        // 3: pLR_s phos-TGFaEGFR_s
// Cytoplasmic species
    this.LR_i = old_molX[4];     // 4: LR_i
    this.TGFa_i = old_molX[5];   // 5: TGFa_i
    this.EGFR_i = old_molX[6];   // 6: EGFR_i
// Nucleus
    this.TGFa_rna = old_molX[7]; // 7: TGFa_rna
    this.EGFR_rna = old_molX[8]; // 8: EGFR_rna
// Feedback attentuation of Migratory signal
    this.PLC = old_molX[9];     // 9: PLCg
    this.PLCt = old_molX[10];   // 10: PLCg_total
    this.PKC = old_molX[11];    // 11: PKC
    this.PKCt = old_molX[12] ;  // 12: PKC_i
    this.Nts = old_molX[13];    // 13: Nts
    this.Gluc = old_molX[14];   // 14: Glucose inside the compartment
    this.Gluc_ex = old_molX[15];// 15: external Glucose

   // this.knet = kRates; //Equating the networks rate constants with
   this.kf  = kRates[0];// = knet[0],       // LR complex formation rate of TGFaEGFR_s and activation (PP) Ls+Rs->LRs
   this.kr  = kRates[1];// = knet[1],       // LR complex dissociation rate of 2xTGFaEGFR_s LRs->Ls+Rs
   this.klri= kRates[2];// = knet[2],     // LR internalization rate
   this.kif = kRates[3];// = knet[3],      // forward reaction of LR_i dissociation LRi->Li+Ri
   this.kir = kRates[4];// = knet[4],      // reverse reaction of LR_i dissociation, Li+Ri->LRi
   this.kls = kRates[5];// = knet[5],      // rate of secretion of TGFa_i
   this.krs = kRates[6];// = knet[6],      // rate of secretion of EGFR_i
   this.kri = kRates[7];// = knet[7],      // rate of secretion of EGFR_i

   this.ktil = kRates[8];// = knet[8],    // TGFa_rna Basal transcription rate
   this.ktir = kRates[9];// = knet[9],   // EGFR_rna Basal transcription rate

   this.Km1 = kRates[10];// = knet[10],    // Km TGFa_rna transcriptional activation
   this.kind1 = kRates[11];// = knet[11],  // kind TGFa_rna transcriptional activation
   this.khil1 = kRates[12];// = knet[12],  // TGFa Hills coeff of induction of transcription
   this.kwh1 =kRates[13];// = knet[13],   // weight of hills coeff

   this.Km2 =kRates[14];//= knet[14],    // Km EGFR_rna transcriptional activation
   this.kind2 =kRates[15];//= knet[15],  // kind EGFR_rna transcriptional activation
   this.khil2 = kRates[16];//= knet[16],  // Hills coeff of induction of transcription
   this.kwh2 =kRates[17];//= knet[17],   // weight of hills coeff

   this.ktral =kRates[18];// = knet[18],  // TGFa_rna rate of translation
   this.ktrar = kRates[19];// = knet[19],  // EGFR_rna rate of translation
   this.krdl = kRates[20];// = knet[20],   // TGFa_rna rate of degradation
   this.krdr =kRates[21];// = knet[21],   // EGFR_rna rate of degradation
   this.kpdl =kRates[22];// = knet[22],   // TGFa_i rate of degradation: TGFa_i --> a.a.
   this.kpdr =kRates[23];// = knet[23],   // EGFR_i rate of degradation: EGFR_i --> a.a.

   this.kpho =kRates[24];// = knet[24],   // min^-1 Phosphorylation of LR_s
   this.kdpho =kRates[25];// = knet[25],  // min^-1 Dephosphorylation of pLR_s

   this.kplca =kRates[26];// = knet[26], // [mol^-1 min^-1] Activation of PLC
   this.kplci =kRates[27];// = knet[27], // [min^-1] In-Activation of PLC
   this.kpkca =kRates[28];// = knet[28], // [mol^-1 min^-1] Activation of PKC-delta
   this.kpkci =kRates[29];// = knet[29], // [min^-1] In-Activation of PLC

   this.Km3    = kRates[30];// = knet[30],      // [M] Km PKC inhibition of pLR*
   this.kfeed1 = kRates[31];// = knet[31],   // [min^-1] kind PKC inhibition of pLR*
   this.kfeedh = kRates[32];// = knet[32],   // Hills coeff PKC inhibition of pLR*
   this.kfeedhw = kRates[33];// = knet[33],  // weight of hills coeff PKC inhibition of pLR*

   this.Km4 =kRates[34];// = knet[34],      // Km of migratory potential
   this.Vm4 =kRates[35];// = knet[35],      // Vmax of migratory potential

   this.Km5 =kRates[36];// = knet[36],      // Km of Proliferative potential
   this.Vm5 =kRates[37];// = knet[37],      // Vmax of Proliferative potential

   this.kglucin =kRates[38];// = knet[38],   // Intake of Glucose from environment
   this.gluc_wt =kRates[39];//= knet[39],   // Weight of Glucose influence on LR phos
   this.wfdgrad =kRates[40];//= knet[40];// Weight of Glucose gradient
   this.kglucMet = kRates[41];//41: Rate of Glucose metabolization

    //those passed to it.
}


  // Compute the no. of molecules of extracellular TGF-alpha protein
   public double computeTGFa_ex() {
     double delta_TGFex =  (kr * TGFaEGFR_s) - (kf * TGFa_ex * EGFR_s)
         + (kls*TGFa_i);// *(1/(1+pLR_s)));
     return delta_TGFex;
   }

// Compute the mass (no. of molecules) of the EGFReceptor protein
  // on the cell surface. This is invoked in each compartment
  public double computeEGFR_s() {
    double delta_EGFRs = (kr * TGFaEGFR_s) - (kf * TGFa_ex * EGFR_s) + (krs*EGFR_i) - (kri*EGFR_s);
         //System.out.println("old L_ex= "+TGFa_ex + " kri= " + kri);
    return delta_EGFRs;
  }

  // Compute the no. of molecules of the ligand-receptor complex
  // EGFR-TGF-alpha on the cell surface
  public double computeTGFaEGFR_s() {
     // PKC inhibition of LR*
     double V3 = (kfeed1 * Math.pow((kfeedhw *  PLC),kfeedh))
         /((Math.pow(Km3,kfeedh)) + (Math.pow((kfeedhw * PLC),kfeedh)));

    double delta_TGFaEGFR_s = (2* kf * TGFa_ex * EGFR_s)
        - (2*kr * TGFaEGFR_s) - (kpho * (TGFaEGFR_s + (TGFaEGFR_s *Gluc*gluc_wt)))
        + (kdpho * pLR_s) - (klri * TGFaEGFR_s) + (V3 * pLR_s);// + (V3* pLR_s);
    return delta_TGFaEGFR_s;
  }


  // Compute the no. of molecules of the ligand-receptor complex
  // EGFR-TGF-alpha on the cell surface
  public double computepLR_s() {
    double V3 = (kfeed1 * Math.pow((kfeedhw *  PLC),kfeedh))
        /((Math.pow(Km3,kfeedh)) + (Math.pow((kfeedhw * PLC),kfeedh)));
    double delta_pLR_s = (kpho * (TGFaEGFR_s + (TGFaEGFR_s *Gluc*gluc_wt)))
        - (kdpho * pLR_s) - (klri * pLR_s) - (V3 * pLR_s);//- (klri*(Gluc/(Gluc+1))*pLR_s);

    return delta_pLR_s;
  }

  // Compute the internalized number of molecules of
  // the ligand-receptor complex EGFR-TGF-alpha on the cell surface
  public double computeTGFaEGFR_i() {
    double delta_LR_i = (klri *(TGFaEGFR_s + pLR_s))
        + (2*kir*TGFa_i* EGFR_i) - (2* kif * LR_i);
    return delta_LR_i;
  }


  // Compute no. of molecules TGF-alpha transcript inside the cell, in the nucleus only
  public double computeTGFa_i() {
    double delta_TGFa_i =  (kif * LR_i) - (kir*TGFa_i*EGFR_i)
        - (kls*TGFa_i)// *(1/(1+pLR_s)))
        + (ktral * TGFa_rna)- (kpdl * TGFa_i);
    return delta_TGFa_i;
  }

  // EGFR_i
  public double computeEGFR_i() {
    double delta_EGFR_i =  (kif * LR_i) - (kir * TGFa_i * EGFR_i)
        - (krs*EGFR_i) + (kri*EGFR_s) + (ktrar * EGFR_rna)- (kpdr * EGFR_i);
    return delta_EGFR_i;
  }

  // TGFa_rna
  public double computeTGFa_rna(){
    //double Nts_c = Nts / (Nav * vol_n);
    //double cpLR_s = pLR_s/(Nav * vol_c);
    double V1 = (kind1 *kwh1* Math.pow(pLR_s,khil1))/((Math.pow(Km1,khil1)) + Math.pow(pLR_s,khil1));
///////////
    double delta_TGFa_rna = (ktil*Nts) - (krdl * TGFa_rna) + V1*Nts;
    return delta_TGFa_rna;
  }
  // %EGFR_rna
  public double computeEGFR_rna(){
    //double Nts_c = Nts / (Nav * vol_n);
    //double cpLR_s = pLR_s/(Nav * vol_c);
    //% EGFR_rna induction
    double V2 = (kind2 *kwh2* Math.pow(pLR_s,khil2))
        /((Math.pow(Km2,khil2))+(Math.pow(pLR_s,khil2)));
//////////
    double delta_EGFR_rna = (ktir*Nts) - (krdr * EGFR_rna) - V2 *EGFR_rna;
  return delta_EGFR_rna;
  }

  // PLC-gamma Active
  public double computePLC() {
    double delta_PLC = (kplca *(PLCt-PLC) * pLR_s) - (kplci * PLC);
    return delta_PLC;
  }


  // PLC-gamma total
  public double computePLCt() {
    double delta_PLCt =0.0;// -(kplca *PLCi *pLR_s) + (kplci * PLC);
    return delta_PLCt;
  }


  // PKC-delta Active
  public double computePKC() {
    double delta_PKC = (kpkca * (PKCt-PKC)* PLC) - (kpkci * PKC);
    return delta_PKC;
  }

  // PKC-delta Inactive
  public double computePKCt() {
    double delta_PKCt = 0.0; //- (kpkca * PKCt* PLC) + (kpkci * PKC);
    return delta_PKCt;
  }

  // Nucleotides in the nucleus
  public double computeNts() {
    double delta_Nts = - (ktir * Nts) - (ktil * Nts)
        + (krdl * TGFa_rna) + (krdr * EGFR_rna);
    return delta_Nts;
  }

  // Glcuose inside the compartment
  public double computeGluci() {
    double delta_gluci = (kglucin * Gluc_ex)
        - (kpho * TGFaEGFR_s *Gluc*gluc_wt) //Glucose (ATP) induced phosphorylation
        -(kglucMet * Gluc);//rate of metabolic consumption of gluci
    return delta_gluci;
  }

// Glucose OUTside the compartment
  public double computeGluc_ex() {
    double delta_gluc_ex = -(kglucin * Gluc_ex); //wfdgrad*(AmAc_ex - AmAc);//THE DIFFERENCE BETWEEN THE
    //FOOD LEVEL INSIDE AND OUTSIDE THE CELL
    return delta_gluc_ex;
  }

  // Compute the force that signals the cell to migrate
  // in a particular direction
  public double migimpulse(double new_PLC) {
    //double cnewpLRs = new_pLRs/(Nav * vol_c);
    //double cnewAAex  = new_AAex/(Nav * vol_c);
    double migForce = (Vm4 * (new_PLC)) / (Km4 + new_PLC);
    return migForce;
  }

  // Compute the force that signals the cell to migrate
  // in a particular direction
  public double prolifpot(double new_pLRs) {
    // double cnewpLRs = new_pLRs/(Nav * vol_c);
    // double cnewgradAAex  = new_gradAAex/(Nav * vol_c);
    double proPot = (Vm5 * (new_pLRs)) / (Km5 + new_pLRs);
    return proPot;
  }

}