package BostonTumor;

import uchicago.src.sim.gui.*; // Import Display2D,SimGraphics,
// DisplayConstants,ViewEvent,DisplayInfo
import java.awt.Color;
import java.util.*; // Import ArrayList, Iterator
//import java.text.DecimalFormat;

/**
 * Title:        Tumor Model Demo 2003
 * Description:  Simulation
 * Copyright:    Copyright (c) 2003
 * Company:      Massachusetts Institute of Technology & Massachusetts General Hospital (Harvard Medical School)
 * @author Yuri Mansury & Chaitanya Athale
 * @version 4.0
 */

public class TumorDisplay
    extends Display2D {

  TumorModel model;
  boolean isShowView = true; // Is this a show view event?
  public TumorDisplay(TumorModel t) {
    super(t.agentTorus); // The constructor of the super class takes a grid
    // object as a 2D display
    this.model = t;
  }

  // The method viewEventPerformed must be implemented
  public void viewEventPerformed(ViewEvent evt) {
    isShowView = evt.showView();
  }

  // The method getDisplayableInfo must be implemented
  public ArrayList getDisplayableInfo() {
    ArrayList list = new ArrayList();
    list.add(new DisplayInfo("View Cell Population", TOGGLE_VIEW, this));
    return list;
  }

  // Implement the abstract method
  public void drawDisplay(SimGraphics g) {
    if (!isShowView) { // If this is not a show view event
      return;
    }
    // Determine the max. pop. of agents per location point: here =1
    int maxPop =1; // if multiple agents per grid pt allowed-- then un-comment the following 2-for-loops
    /*
    for (int i = 0; i < grid.getSizeX(); i++) {
      for (int j = 0; j < grid.getSizeY(); j++) {
        if (model.agentTorus.getCellSizeAt(i, j) > 0){// maxPop) {
          maxPop =1;// model.agentTorus.getCellSizeAt(i, j);
          //in our case maxPop never exceeds 1!!
        }
      }
    }*/

    //determines the maximal food concentration
    double maxGluc = 0.0;
    for(int i = 0; i < model.numLocations; i++){
      if (model.foodLevel[i] > maxGluc) {
        maxGluc = model.foodLevel[i];//get maximum food level
      }
    }

    //  determines the maximal extra-cellular TGF
    double maxETGFa = 0.0;
    for(int j = 0; j < model.numLocations; j++){
      if (model.autocrFood[j] > maxETGFa) {
      	maxETGFa = model.autocrFood[ j ];//get maximum food level
      }
    }



// Determine the location with max. pop. of dead agents
//    int maxDeadPop = 0 ;
//    if(model.showDeadCell) {
//      for(int i = 0 ; i < model.numLocations ; i++) {
//        if(model.deadCell[i] > maxDeadPop)
//          maxDeadPop = model.deadCell[i] ;
//      }
//    }
    // Determine the location with max. pop. of migrating agents
    //(FOR CASE WHEN MORE THAN ONE AGENT PER GRID POINT)
    /*int maxMigrPop = 0;
    if (model.showMigrCell) {
      for (int i = 0; i < model.numLocations; i++) {
        if (model.migrCell[i] > maxMigrPop) {
          maxMigrPop = model.migrCell[i];
        }
      }
    }

    // Determine the location with max. pop. of migrating agents
    int maxQuiescPop = 0;
    if (model.showQuiesCell) {
      for (int i = 0; i < model.numLocations; i++) {
        if (model.quiescCell[i] > maxQuiescPop) {
          maxQuiescPop = model.quiescCell[i];
        }
      }
    }

    // Determine the location with max. pop. of migrating agents
    int maxProPop = 0;
    if (model.showProCell) {
      for (int i = 0; i < model.numLocations; i++) {
        if (model.proCell[i] > maxProPop) {
          maxProPop = model.proCell[i];
        }
      }
    }*/

    // Now we determine the color "intensity" of each grid cell depending
    // on the on-site population density relative to maxPop:
    int size = 0, sizeMigr = 0, sizeQuiesc = 0, sizePro = 0; 
         //sizeDead = 0,
    float unit = 0.99f; // The highest color "depth" possible
    double colorIn=0.0, colorInGluc = 0.0, foodPeakLevel=0.0, gluclevel=0.0, tgflevel=0.0, autoPeakLevel=0.0,
    colorInAuto = 0.0;// color intensity of food
    int distpeak = 0;

    //  if (maxDeadPop > 0)
    //    logMaxDead = Math.log((double)maxDeadPop) ;
    /*if (maxMigrPop > 0) {
      logMaxMigr = Math.log( (double) maxMigrPop);
    }
    if (maxPop > 0) {
      logMaxPop = Math.log( (double) maxPop);
    }
    */  
    int xsour = Math.abs(model.resolution * 3 / 4);
    int ysour = Math.abs(model.resolution / 4);
    int[] xysour = {xsour,ysour};

    for (int i = 0; i < grid.getSizeX(); i++) {
      for (int j = 0; j < grid.getSizeY(); j++) {
      	//unique identifier in a 2D grid
        int migrID = i * model.resolution + j;

        // Set the drawing coordinates of each grid cell location:
        g.setDrawingCoordinates(i * DisplayConstants.CELL_WIDTH,
                                j * DisplayConstants.CELL_HEIGHT, 0);
        size = model.agentTorus.getCellSizeAt(i, j);
        if (size > 0) {
            //double logSize = Math.log( (double) size);
            //colorIn = 0.2 + 0.8 * (logSize / logMaxPop);
          	double popSize = 1.0;
          	colorIn = 1.0;
          }
          else {
            colorIn = 0.0;
          }

        //////////Making glucose level differences visible
          gluclevel = model.foodLevel[migrID];
          if (maxGluc > 0) { // Test if maxGluc is a positive number
              //double logGluc = Math.log(gluclevel);
              colorInGluc = 255 * gluclevel / maxGluc;
          }
          else {
            System.out.println("The max-Glucose level is NEGATIVE!\n");
          }
//////////Making TGFa level differences visible
          tgflevel = model.autocrFood[migrID];
          if (maxETGFa > 0) { // Test if maxGluc is a positive number
              //double logGluc = Math.log(gluclevel);
              colorInAuto = 255 * tgflevel / maxETGFa;
          }
          else {
            System.out.println("The max. ext. TGFa level is NEGATIVE!\n");
            model.autocrFood[migrID]=0;
            colorInAuto = 0;
          }

          
        //////////
          //DEAD CELLS
          // if(model.showDeadCell) {
          //   int deadID = i*model.resolution + j ;
          //   colorInDead = 0.0 ;
          //   sizeDead = model.deadCell[deadID] ;
          //   if(logMaxDead > 0) {
          //     if(sizeDead > 0) {
          //       double logDead = Math.log((double)sizeDead) ;
          //       colorInDead = 0.2 + 0.8*(logDead/logMaxDead) ; }
          //     else
          //       colorInDead = 0.0 ; }
          // }

    ////// MIGRATING CELLS
    sizeMigr = model.migrCell[ migrID ];
    ////// Quiescent cells CELLS
    sizeQuiesc = model.quiescCell[ migrID ];    
    ////// Proliferating CELLS
    sizePro = model.proCell[ migrID ];
        //if(model.isFoodPeakshown){
        int[] xygrid = {i, j};
        distpeak = model.computeEuclD(xysour,xygrid);  //EUCLIDEAN DISTANCE
        //this provides a circular food source in the display 
        //System.out.println("euclidean distance from the peak= "+distpeak);
        // if(distpeak >= model.foodPeakRange)
        foodPeakLevel = (double)(model.foodLevel[migrID]);// foodLevel[]/ model.foodLevelMax[migrID]);
        autoPeakLevel =(double)(model.autocrFood[migrID]);
         

        try {
          // Draw a rectangle with specified RGB (red, green, blue) intensity
          //  if (!model.showMigrCell) {
          //    g.drawFastRect(new Color( (float) colorIn, 0.0f, 0.0f));
          //  }

          if (size > 0 && sizePro == 1) {
          	  g.drawFastRect(Color.YELLOW);
           }
          else if (size > 0 && sizeMigr == 1) {
          	g.drawFastRect(Color.WHITE);
          }
          else if (size > 0 && sizeQuiesc == 1){
          	g.drawFastRect(Color.BLUE);
          }
         //Food PEAK
         // THE peak in NE quadrant
          else if (model.isFoodPeakshown) {
          	//draw the edge of the cross-sectional blood vessel
          	if (foodPeakLevel > 0 && size == 0 &&
              (distpeak == model.foodPeakRange)) {//ONLY edge of source
          	  //(distpeak < model.foodPeakRange)) { WHOLE SOURCE
          	  //IF the food is scaled to intensity of glucose
              g.drawFastRect(Color.RED); //(float) colorInGluc, (float) colorInGluc, (float) colorInGluc));
            }
            //--- display the Gluc. Diffusing wave
            else if (foodPeakLevel > 0 && size == 0) {//diffusing wave where no cells exist
              g.drawFastRect(new Color( (float) ( 0 / 255.0),
                                       (float) (colorInGluc / 255.0),
                                       (float) (0 / 255.0)));
              }
//          

            else if(size == 0 && foodPeakLevel == 0){
            	g.drawFastRect(Color.BLACK);//background
            }
          }
          else{// if(size == 0){//background
          	//---e-TGFa distr
            if (foodPeakLevel == 0 && size == 0 && autoPeakLevel > 0) {//diffusing wave where no cells exist
              g.drawFastRect(new Color( (float) ( 0 / 255.0),
                                       (float) (colorInAuto*0.5 / 255.0),
                                       (float) (colorInAuto / 255.0)));
              }
          	else 
          		g.drawFastRect(Color.BLACK);
          }
            // IF FOOD IS SHOWN WITHOUT GRADIENT
            //  g.drawFastRect(new Color(Color.GRAY));
          //  if ((distpeak == model.foodPeakRange+10&& size == 0)) {
          //ONLY edge of source + 10
           //g.drawFastRect(new Color( (float) colorInGluc, 0.5f,0.0f));
           //}

          if ( (i == 0) || (i == (model.resolution - 1)) || (j == 0) ||
              (j == (model.resolution - 1))) {
            g.drawFastRect(Color.BLACK);
          }
        }
        catch (java.lang.IllegalArgumentException e) {
          System.out.println("At drawDisplay, Error: IllegalArgumentException!");
        }
        // If desired, display the size of the population on the grid
        /*if ( (size > 0) && ( (TumorModelGUI) model).isSizeShown) {
          g.drawString("" + size, Color.black);
        }
        if ( (sizeMigr > 0) && ( (TumorModelGUI) model).isMigrCellShown) {
          g.drawString("" + sizeMigr, Color.black);
        }
        */
      }
    }
  }

}
