package BostonTumor;

import uchicago.src.sim.engine.*; //import SimpleModel, SimInit
import uchicago.src.sim.space.*; //import Multi2DTorus
import java.util.*; //import Iterator, ArrayList
import java.io.*; // import file input output classes
import java.awt.Dimension; //import dimension class
import java.lang.Math; //import floor

/**
 * Title:        Tumor Model Demo 2003
 * Description:  Network + Compartment Version 2D, 1 cell per grid pt
 * Copyright:    Copyright (c) 2003
 * Company:      Massachusetts General Hospital
 * @author Chaitanya Athale
 * @version 4.0h
 */

public class TumorModel
    extends SimpleModel {

  protected long startTime; // Starting time
  protected double elapsedTimeSecond; // Elapsed time in seconds
  Multi2DTorus agentTorus; //The 2D multi torus of agents
  protected int resolution; //The size of the 2D grid's "sides"
  protected int numLocations; //The total number of locations on the grid

  protected int agentSize; //Initial number of agents
  protected int agentID; //Keeping track of agent ID's
  protected int agentReach; //The max. distance that a tumor cell can reach
  protected int numNeighbors; //The number of neighbors that a location has
  protected int numNewAgent; //The number of newly-generated agents/daughter cells
  protected int totalCompartments; //Total number of subcellular compartments per cell

  int numMovers; //User-specified max. number or percent of moving agents
  boolean isResistCrater; //Is there a crater of resist at origin & random elsewhere
  boolean isFoodPeak; //Is there a peak of food at Quadrant 3 & random elsewhere
  boolean isOrigRep; //Is the area around origin at Quadrant 1 replenished
  boolean isFoodRandom; //Is food distributed randomly

  int resistCraterRange; //Range (radius) of resistance crater at Quadrant 1
  int foodPeakRange; //Range (radius) of food peak at Quadrant 3
  int threshLagPro; //lag before a cell divides once it has the signal
  int threshLagMig; // lag per step before the cell migrates

  int printallmol; //print molecular values after sim 0=no, 1=yes
  int avgsingle; //calculate average or single cell values for molec profiles

  int[] numSwitch; //keeps track of how many cells are changing phenotypes
  int totnSwitch; //Keeps track of total number of cells that change their phenotypes
  int totAg; //total number of agents
  //0 = from migratory (P) to proliferative (M) states
  //1 = from migratory (M) to proliferative (P) states
  //2 = from quiescent (Q) to pro (P) or mig (M) states

  double rRho; //Intrinsic capability to migrate
  double congest; //Parameter congestion/overcrowding ("c" in paper)
  double spill; //The extent of spillovers from neighboring locations
  double goodSigma; //Standard deviation of food distribution
  double resistanceSigma; //Standard deviation of resistance distribution

  double qFood; //Food coefficient in utility
  double qToxic; //Toxic coefficient in local utility
  double qResistance; //Resistance coefficient in local utility

  double rFood; //The rate of Glcuose updake (depletion from grid)
  double rToxic; //The rate of toxic accumulation
  double rResistance; //The rate of resistance reduction
  double gFood; //The (grow-back) rate of food production
  double wFood; //The weight of food

  double kFoodiff; //flux rate due to diffusion
  double kAutodiff; //flux rate due to diffusion
  double rTgfexDeg; // rate of TGFa external degradation
  double kProlif; //Parameter controlling the probability to proliferate
  double kDeath; //Parameter controlling the probability of death

  double threshPLC; //Threshold parameter for PLCg active concentration
  double threshPLCnoise; //noise threshold for PLCg active
  double threshpLRprol; //noise threshold for PLCg active
  double Dnasyn; //Amount of DNA synthesized
  double threshDnasyn; //Threshold for cell to divide if synthesized DNA exceeds it

  double errPLC; //error discrimination in PLC concentration
//  double errfactPLC; //error factor in PLCg based direction sensing

  double foodProlifThreshold; //Min. food level for prolif. to occur
  double searchPrecision; //How precise (error-free) search is

  double coreTGFrna; //value of total TGFa RNA in core cells
  double edgeTGFrna; //value of total TGFa RNA in edge cells
  double coreEGFRrna; //value of total EGFR RNA in core cells
  double edgeEGFRrna; //value of total EGFR RNA in edge cells
  double coreEGFRs; //value of total EGFR RNA in core cells
  double edgeEGFRs; //value of total EGFR RNA in edge cells

  double[] initAgentloc; //Array to store initial location of agents
  double[] foodLevel; //Array that stores the level of food
  double[] oldfoodLevel; //Array with the old food levels
  protected double[] foodLevelMax; //Array that stores max. level of food
  double[] resistanceLevel; //Array that stores the level of resistance
  protected double[] resistLevelMax; //Array that stores max. level of resist.
  double[] toxicLevel; //Array that stores the level of toxicity
  int[] deadCell; //Array that keeps track of dead cells
  int[] migrCell; //Array that keeps track of migrating cells cells
  int[] quiescCell; //Array that keeps track of quiescent cells
  int[] proCell; //Array that keeps track of proliferating cells
  int[] foodPeakLoc; //Array keeps track of the food peak location
  //Array keeps track of the number of occupied boxes in fractal dim estimation
  double[][] boxList;
  //Array stores the values of the linear fitting routine
  double[] fitvals;
  double[][] xydata;
  //Arrays for measuring trajectories
  int[][] xyAgents;

  //Array stores average compartmental values of migrating and proliferating cells
  double[] migCellav;
  double[] proCellav;
  double[] migCellPLC;
  //Array for dist of agent from centre
  double[] Roughness;
  double[] distcent;
  double StdevDist; //Std. Deviation of cells at the edge of the tumor
  double Avgdist; // average distance of edge cells from the centre of the grid

  double avAgdist; //average distance between agents- migrating- 3Quadrant
  int numMigAg3q; //number of migrating agents in the third quadrant

  //DFA related parameters
  double Migpro;
  double Timept;
//  double[] RatioMigpro;

//initialize diffused food
  double[] diffusFood; //temporary array sotring the values of food as it diffuses
//initializing the array to hold values output by the network
  double[] oldAutocrFood; //secreted food from individual agents
  double[] autocrFood; //secreted food from individual agents
  double[] diffAutocr; //temporarily stores diffusing autocrine food

  double TGFexi; //Initial TGFex distribution: for use in TumorModelBatch: compt 0
  double initEGFRs; //Initial TGFex distribution: for use in TumorModelBatch: compt 0

  double rLRint; // Rate of ligand receptor internalization
  double indTGFrna; // Rate of TGFa rna transcription induction (strength)
  // Declaring values to be later output
  double migTGFrna, //TGF_rna- in each compartment- migrating cell
      migEGFRrna,
      proTGFrna, //proliferating
      proEGFRrna; //EGFR_rna - migrating cell
  double[] migTGFaex, //TGFa ex protein- migrating cell
      migPpLR, // pp TGFa-EGFR_s protein- migrating cell
      migPLCg, // PLCgamma protein - migrating cell
      proTGFaex,
      proPpLR,
      proPLCg;

  // Lookup table stores the max. x coord. reachable from an x location:
  int[] xLowerBound, xUpperBound;
  // Lookup table stores the max. y coord. reachable from a y location:
  int[] yLowerBound, yUpperBound;

  // Boolean keeping track whether cells have reached the center of Quadrant 3:
  boolean isThirdQuadrantReached;
  // Variable recording the time when the center of Quadrant 3 is first reached:
  long timeToThirdQuadrant;
  boolean isDeadCellShown; // On/off switch to display dead cells
  boolean showDeadCell;

  boolean isMigrCellShown; // On/off switch to display migrating cells
  boolean showMigrCell;
  boolean isQuiescCellShown; //on/off switch to display quiescent cells
  boolean showQuiesCell;
  boolean isFoodPeakshown; // On/Off to switch the display of the food peak
  boolean isProCellShown;
  boolean showProCell;

  // Array list containing moving agents
  ArrayList movingList;
  // Array list containing proliferating agents
  ArrayList proliferateList;
  // Array list containing dying agents
  //ArrayList deathList;
  //Array list containing agents at the core of the spheroid
  ArrayList coreList;
  //Array list containing agents at the edge of the spheroid
  ArrayList edgeList;
  //Array list containing agents at that proliferate
  ArrayList prolifList;
  //Array list containing agents at that migrate
  ArrayList migList;
  //ArrayList with cells that are quiescent
  ArrayList quiescList;

  // A text file for maitaining the list of molecules in a cell
  FileWriter fw;

  // Initializing the model /////////////////////////////////////////////
  public void setup() {
    // The setup() method of the parent MUST be called first!
    super.setup();

    startTime = System.currentTimeMillis(); // Starting time
    elapsedTimeSecond = -1.0; // Elapsed time in seconds
    setRngSeed(1); // Set the seed for the random number generator

    resolution = 200;
    agentSize = 100; // Use 10 for 50^2 grid // AT INITialization
    agentReach = 1;
    numNeighbors = 9; // The number of Moore neighbors including itself
    numNewAgent = 0; // The number of newly-generated daughter cells
    totalCompartments = 4; //Number of compartments per agent

    TGFexi = 0.1; //15;//Initial TGFex distribution: for use in TumorModelBatch: compt 0
    initEGFRs = 329; //6//31.2//329 Initial concentration of EGFR surface receptors
    rLRint = 0.2; //0.001;
    indTGFrna = 0.5; // standard wt of induction by activated TGFa-EGFR

    threshLagPro = 10;
    threshLagMig = 1;

    printallmol = 0; //print mol conc = 1; dont print=0
    avgsingle = 0; //1=print the average values; 0=vals of single cells

    coreTGFrna = 0;
    edgeTGFrna = 0;
    coreEGFRrna = 0;
    edgeEGFRrna = 0;
    coreEGFRs = 0;
    edgeEGFRs = 0;

    proCellav = new double[4];
    migCellav = new double[4];
    migCellPLC = new double[4];

    numSwitch = new int[4];
    totnSwitch = 0;
    totAg = 0;

    migTGFrna = 0; //TGF_rna- in each compartment- migrating cell
    migEGFRrna = 0; //EGFR_rna - migrating cell
    migTGFaex = new double[4]; //TGFa ex protein- migrating cell
    migPpLR = new double[4]; // pp TGFa-EGFR_s protein- migrating cell
    migPLCg = new double[4]; // PLCgamma protein - migrating cell
    proTGFrna = 0.0; //proliferating
    proEGFRrna = 0.0;
    proTGFaex = new double[4];
    proPpLR = new double[4];
    proPLCg = new double[4];
//Array to keep track of number of time pt, agentID, ag-X and ag-Y coords.
    //agTrack = new double[agentSize][4];

    numMovers = 5000; // 10 ; // Setting the default for max. number/% of movers
    isResistCrater = true; // Default for configuration of resistance
    isFoodPeak = true; // Default for configuration of food
    isFoodRandom = false; // Default for distribution of food
    isOrigRep = false; // Default for food replenishment at origin Quadrant 1
    foodPeakRange = resolution / 10; // Default of radius of food peak
    //System.out.println("At setup,foodPeakRange=" + foodPeakRange);
    //resistCraterRange = 0; //10 foodPeakRange / 16; // Default of resistance-crater range
    //System.out.println("At setup,resistCraterRange=" + resistCraterRange);

    rRho = 0.0;
    congest = 0.1;
    spill = 0.5;
    goodSigma = 8.0; // Rule of thumb: use 8.0 for 50^2,48 for 300^2
    if (isFoodPeak) {
      goodSigma = 10;
    }
    // For resistanceSigma, same rule of thumb as food, but make it differ
    // slightly from goodSigma IF no randomness in the generation of either:
    resistanceSigma = 8.5; // Rule of thumb: use 8.5 for 50^2,51 for 300^2
    if (isResistCrater) {
      resistanceSigma = 3;
    }
    qFood = 10.0; // Coefficient for food in utility function
    qToxic = 10.0;
    qResistance = 10.0;
    rFood = 0.07; //SAME AS kglucin 0.01; // Rate of food depletion
    rToxic = 0.0; //#CHANGE0.02;
    rResistance = 0.001;
    gFood = 0.00; // Grow back rate of food production
    wFood = 0.01; //CHANGED#5.0 * 90.0; // The weight of food

    kFoodiff = 1 * 10 ^ ( -3); //flux rate due to diffusion
    kAutodiff = 1 * 10 ^ ( -7); ; //1*10^(-12); //flux rate of TGFa external due to diffusion
    rTgfexDeg = 1 * 10 ^ ( -2); //rate of TGFa degradation

    kProlif = 2; //#CHANGED200.0; // Controlling the probability to proliferate
    kDeath = 0.0; //10.0; // Controlling the probability to die

    threshPLC = 0.0008; //0.0008;//!!!! 0.0009; //threshold of PLCg active below which prolif and above migration
    threshPLCnoise = 0.000; //threshold above noise of PLCg active conc.
    threshpLRprol = 0.0002; //threshold of phos-TGFaEGFR to permit proliferation
    //System.out.println("Prol/Mig Thresh= "+ threshPLC);
    threshDnasyn = 0.0;
    errPLC = 0.3; //error tolerance to PLC conc differences from threshold
    // this value of errPLC is a fraction of the maxPLC for that cell
    // i.e. a relative value. maxPLC itself is a relative and adaptive way of deciding
    // cell fate, alternative to hard coding a threshold.

    // foodProlifThreshold =0.1;// 5.0 * 90.0 * 0.0001; // Min. food level to proliferate
    searchPrecision = 0.7; // Between 1 = error-free and 0 = max. error
    if (searchPrecision > 1.0) {
      //System.out.println("At setup, searchPrecision cannot be greater than 1.0");
      searchPrecision = 1.0;
      //System.out.println("At setup, setting searchPrecision to=" +
      //                searchPrecision);
    }

    isThirdQuadrantReached = false; // Initially there is no cell in Quadrant 3
    timeToThirdQuadrant = -1;
    //OK:
    isDeadCellShown = false; // Default for whether dead cells are shown
    isMigrCellShown = true; // Default for whether migrating cells are shown
    isQuiescCellShown = true; //Default for whether quiescent cells are shown
    isProCellShown = true;
    isFoodPeakshown = true;// true; //Default for showing the food peak location

    // See the documentation for the class SimpleModel:
    autoStep = true;
    shuffle = true;

  }

  // The method to build the Model's internals //////////////////////////
  public void buildModel() {
    // Creating the torus
    agentTorus = new Multi2DTorus(resolution, resolution, false);
    numLocations = resolution * resolution;

    initAgentloc = new double[numLocations];
    foodLevel = new double[numLocations];
    oldfoodLevel = new double[numLocations];
    foodLevelMax = new double[numLocations];
    resistanceLevel = new double[numLocations];
    resistLevelMax = new double[numLocations];
    toxicLevel = new double[numLocations];
    autocrFood = new double[numLocations];
    oldAutocrFood = new double[numLocations];
    diffusFood = new double[numLocations];
    diffAutocr = new double[numLocations];
//Fractal dimension fitting value
    fitvals = new double[3];

    Roughness = new double[2];
    distcent = new double[1];
    double StdevDist = 0.0; //Std. Deviation of cells at the edge of the tumor
    double Avgdist = 0.0; // average distance of edge cells from the centre of the grid

    double avAgdist = 0.0;
    int numMigAg3q = 0;

    Migpro = 0.0;
    Timept = 0.0;
//    double[] RatioMigpro = new double[10000];

    showDeadCell = isDeadCellShown; // Necessary to avoid on-the-fly turn on
    if (showDeadCell) {
      deadCell = new int[numLocations];
      for (int i = 0; i < numLocations; i++) {
        deadCell[i] = 0;
      }
    }
//array of migrating cells- for DISPLAY
    showMigrCell = isMigrCellShown; // Necessary to avoid on-the-fly turn on
    if (showMigrCell) {
      migrCell = new int[numLocations];
      for (int j = 0; j < numLocations; j++) {
        migrCell[j] = 0; //initialize the migrCells[]
      }
    }

//array of quiescent cells- for DISPLAY
    showQuiesCell = isQuiescCellShown; // Necessary to avoid on-the-fly turn on
    if (showQuiesCell) {
      quiescCell = new int[numLocations];
      for (int j = 0; j < numLocations; j++) {
        quiescCell[j] = 0; //initialize the migrCells[]
      }
    }
//array of proliferating cells- for DISPLAY
    showProCell = isProCellShown;
    if (showProCell) {
      proCell = new int[numLocations];
      for (int j = 0; j < numLocations; j++) {
        proCell[j] = 0; //initialize the migrCells[]
      }
    }

    initTumorCells(); // Call the method to initialize tumor cells
    initFood(); // Call the method to initialize food level

    // Initialize the lookup tables
    xLowerBound = new int[resolution];
    xUpperBound = new int[resolution];
    yLowerBound = new int[resolution];
    yUpperBound = new int[resolution];

    for (int i = 0; i < resolution; i++) {
      xLowerBound[i] = i - agentReach;
      yLowerBound[i] = i - agentReach;
      xUpperBound[i] = i + agentReach + 1;
      yUpperBound[i] = i + agentReach + 1;
    }

    // Initializing the list of moving agents
    movingList = new ArrayList();
    migList = new ArrayList();
    prolifList = new ArrayList();

  }

  // The method that initializes tumor cells
  public void initTumorCells() {

    // Place the Each new agent symmetrically around the center
    // Resolution - 0.5 x sqrt(agentSize) gives x and y starting points
    // OK: Chaitanya: this is an off by one problem // (one too many initial agents)

    int agLeng = (int) (Math.sqrt(agentSize)) + 1;
    int initLocs = (int) (resolution - agLeng) / 2,
        initLoce = (int) (resolution + agLeng) / 2;
    agentID = 0; // The ID of the agent under creation
    int locID = 0;
    for (int xl = initLocs; xl < initLoce; xl++) {
      int xll = agentTorus.xnorm(xl);
      for (int yl = initLocs; yl < initLoce; yl++) {
        int yll = agentTorus.xnorm(yl);
        locID = xll * resolution + yll;
        //for (int i = 0; i < agentSize; i++) {}
        if (agentID < agentSize) {
          Agent a = new Agent(agentID, this); // Create an agent
          for (int ncompt = 0; ncompt < 4; ncompt++) {
            a.Xmol_ia[ncompt][0] = TGFexi;
            a.Xmol_ia[ncompt][1] = initEGFRs;
          }

          //System.out.println("TGFex " + a.Xmol_ia[0][0]);
          a.placeTo(xll, yll);
          agentList.add(a); // Add the newly created agent to the list of agents
          agentID++; // Increment the identity counter

        } //only if the initial agents  havent yet been placed
        else {
          continue;
        }
      }
    }
    //System.out.println("No. of initial agents 0-"+(agentID));
  }

  // The method that initializes food level
  public void initFood() {
    //OK: Chaitanya: check if food is ONLY available at the peak
    int locID = 0; // The array index corresponding to the food position
    int x, y, xOrig, yOrig;
    int dist, distOrig; //L-infinity measure of distance
    int edist, edistOrig; //euclidean distance measures
    int[] Quad3 = {
        Math.abs(resolution * 3 / 4),
        Math.abs(resolution / 4)};
    int[] OrigGrid = {
        0, 0};

    double weight, weightOrig;
    for (int ii = 0; ii < resolution; ii++) {
      int i = agentTorus.xnorm(ii);
      for (int jj = 0; jj < resolution; jj++) {
        int j = agentTorus.ynorm(jj);
        locID = i * resolution + j; // Retrieve the ID of this location
        x = Math.abs(resolution * 3 / 4 - i);
        y = Math.abs(resolution / 4 - j);
        int[] Locxy = {
            i, j};
        dist = computeDistance(x, y); // Distance from center of Quadrant 3
        edist = computeEuclD(Quad3, Locxy);
        xOrig = Math.abs(resolution / 2 - i);
        yOrig = Math.abs(resolution / 2 - j);
        distOrig = computeDistance(xOrig, yOrig); // Distance from origin
        weight = Math.exp( -Math.pow( (double) dist, 2.0) /
                          (2.0 * Math.pow(goodSigma, 2.0)));
        weightOrig = Math.exp( -Math.pow( (double) distOrig, 2.0) /
                              (2.0 * Math.pow(goodSigma, 2.0)));

        // Set initial foodlevels = 0
        foodLevel[locID] = 0.0;
        oldfoodLevel[locID] = 0.0;

        if (isFoodPeak) {
//          if (dist <= foodPeakRange) { // Food peaks at center of Quadrant 3
          if (edist <= foodPeakRange) { // Food peaks at center of Euclidean Quadrant 3
            if (isFoodRandom) {
              foodLevel[locID] = 2 * this.wFood *
                  getNextDoubleFromTo(0.75, 1.0) * weight;
            }
            else if (!isFoodRandom) {
              //since we use non random food, we get this actual value for peak food
              foodLevel[locID] = 10 * this.wFood *
                  getNextDoubleFromTo(0.5, 1.0) * weight;
            }
            foodLevelMax[locID] = foodLevel[locID];

          }
          else {
            foodLevel[locID] = 0.0;
          }
          foodLevelMax[locID] = 5.0;
        }
        else {
          foodLevel[locID] = 0.0;
        }
        //for all other locations
        //Glucose/food levels are 0.0
        foodLevelMax[locID] = 5.0; // Set max. food level
      }
    }
    oldfoodLevel = foodLevel;
  }

  //Method to compute distance from the origin
  public int computeDistance(int x, int y) {
    int distance;
    x = ( (double) x > (double) resolution / 2.0) ? resolution - x : x;
    y = ( (double) y > (double) resolution / 2.0) ? resolution - y : y;
    //If x > resolution/2, because space is a torus, the "true" x
    //is resolution - x.
    distance = (x > y) ? x : y; //L-infinity metric of distance
    return distance;
  }

  //Method to compute EUCLIDEAN distances from any given point
  public int computeEuclD(int[] point1, int[] point2) {
    double euclDist;
    euclDist = Math.sqrt(Math.pow( ( (double) (point1[0] - point2[0])), 2.0) +
                         Math.pow( ( (double) (point1[1] - point2[1])), 2.0));
    euclDist = Math.abs(euclDist);
    return (int) euclDist;
  }

  // The pre-step method
  public void preStep() {
    // First, we reset the flags for every agent
    Iterator it = agentList.iterator();
    while (it.hasNext()) {
      Agent a = (Agent) it.next();
      a.reset();
    }
    proliferate(); // Marking which agents will proliferate, however actual
    // proliferation is done later
    diffuseFood(); // Causes food to diffuse down the gradient
    updateAutocr(); //causes old autocrine TGFa grid to be updated
    updateFood(); // Update the level of food

    // Creation of new cells is implemented in postStep()
    // Setting the counter for limiting the number of moving agents
    Agent.alreadyMoved = 0;
    movingList = new ArrayList(); // Reset the list of moving agents
    migList = new ArrayList();
    prolifList = new ArrayList();
    //migrCell = new int[numLocations];//reset migrCell
    //for (int j = 0; j < numLocations; j++) {
    //   migrCell[j] = 0;//initialize the migrCells[]
    //  }

  }

  // The method that updates the level of food in every location
  public void updateFood() {
    //OK: Chaitanya: please make sure that REPLENISHMENT takes place !!

    // if(foodLevel[21075] > 0){
    //   System.out.println("Gluc wave reaches centre conc= " + foodLevel[20100]);
    // }
    int[] Quad3 = {
        Math.abs(resolution * 3 / 4),
        Math.abs(resolution / 4)};
    int[] OrigLoc = {
        Math.abs(resolution * 2), Math.abs(resolution / 2)};

    int locID = 0;
    for (int ii = 0; ii < resolution; ii++) {
      int i = agentTorus.xnorm(ii);
      for (int jj = 0; jj < resolution; jj++) {
        int j = agentTorus.xnorm(jj);

        locID = i * resolution + j; // Retrieve the ID of this location
        int[] xyLoc = {
            i, j};
        // Update first if the current level of food is less than the max.
        //     if (foodLevel[locID] < foodLevelMax[locID]) {
        //here food is replenished at the rate of gFood
        double food = foodLevel[locID];
        // (1 + gFood) * (foodLevel[locID]+ foodLevelMax[locID]) /2;
        //      if (food > foodLevelMax[locID]) {
        //        food = foodLevelMax[locID]; // foodLevel cannot exceed the max.
        //      }
        if (isFoodPeak) {
          int x = Math.abs(resolution * 3 / 4 - i),
              y = Math.abs(resolution / 4 - j);//coordinates of food-peak
          //System.out.println("foodPeak x: "+x+", y: "+y);
          int dist = computeDistance(x, y); // Distance from center of Quadrant 3
          int edist = computeEuclD(xyLoc, Quad3);
          //if (dist <= foodPeakRange) { // Replenish around center
          if (edist <= foodPeakRange) { // Replenish around center
            if (food > foodLevelMax[locID]) {
              food = foodLevelMax[locID]; // foodLevel cannot exceed the max.
            }
            else {

              //if (foodLevel[locID] < foodLevelMax[locID]) {
              food = (1 + gFood) * (foodLevel[locID] + foodLevelMax[locID]) / 2;
              // Replenish the peak around the origin
              //}
              //System.out.println("locID "+locID+" foodPeak food= "+food);
            }
            if (food > foodLevelMax[locID]) {
              food = foodLevelMax[locID]; // foodLevel cannot exceed the max.
            }
          }
          if (isOrigRep) {
            int xOrig = Math.abs(resolution / 2 - i),
                yOrig = Math.abs(resolution / 2 - j);
            int distOrig = computeDistance(xOrig, yOrig);
            int edistOrig = computeEuclD(xyLoc, OrigLoc);
            //if ( (distOrig >= resistCraterRange) && dist > foodPeakRange) {
            if ( (edistOrig >= resistCraterRange) && dist > foodPeakRange) {
              foodLevel[locID] = (1 + gFood) *
                  (foodLevel[locID] + foodLevelMax[locID]) / 2; //food;
            }
          }
          else {
            food = foodLevel[locID];

          }
        }
        else if (!isFoodPeak) {
          // The current food stock equals previous level plus current production
          foodLevel[locID] = food; // Set the new (gross) foodLevel
        }
        foodLevel[locID] = food;
        // }
        //////INPUT of FOOD FROM AGENT to FOOD GRID:for CA
        // Now further update only this site is non-empty:
        /*  if (agentTorus.getCellSizeAt(i, j) > 0) {
            // Compute the increment decline of food on this site
            double deplete = rFood * (double) agentTorus.getCellSizeAt(i, j);
            if (foodLevel[locID] > deplete) {
              foodLevel[locID] -= deplete; // Update the (net) level of food
            }
            else { // The level of food cannot be lower than zero
              foodLevel[locID] = 0.0;
            }
          }
         FOOD DEPLETION done in agent computeNetwork
         */
//        if(getTickCount() == 80){//FOR 3D PLOTTING
//          if(foodLevel[locID] > 0)
//            System.out.println("At X, "+i+", Y, "+j+", foodLevel=, "+ foodLevel[locID]);
//        }
      }
    }
    oldfoodLevel = foodLevel;

    //System.out.println("update old food 1");
  }

// Simulate Diffusion of the food material through the grid
  public void diffuseFood() {
    int locID = 0;
    diffusFood = oldfoodLevel;
    diffAutocr = oldAutocrFood;
    double foodout = 0, autocrout = 0; //outflow
    int cellLoc = 0; //number of cells at a given location
    //diffAutocr =  oldAutocrFood;
    for (int locx = 0; locx < resolution; locx++) {
      int locxd = agentTorus.xnorm(locx);
      for (int locy = 0; locy < resolution; locy++) {
        int locyd = agentTorus.ynorm(locy);
        locID = locxd * resolution + locyd; // Retrieve the ID of this location

        if (locxd > 1 && locxd < resolution - 1 && locyd > 1 &&
            locyd < resolution - 1) {

          double nbFoodgrad = 0, avgnbGluc = 0; //gradient of food between one grid point and Neighbour
          double nbAutocgrad = 0, avgnbTGFa = 0;
          int pp = 0;
          //int locID = locx * resolution + locy; // Retrieve the ID of this location
          int dxLo = xLowerBound[locxd], dxHi = xUpperBound[locxd];
          int dyLo = yLowerBound[locyd], dyHi = yUpperBound[locyd];
          int locnbID = 0, ncells = 0;
          double foodin = 0, autocrin = 0; //inflow

          int cellnum = 0;
          cellLoc = agentTorus.getCellSizeAt(locxd, locyd);
          //if(locxd==100 && locyd==100){
          //  System.out.println("loc CELL :x " + locxd + " x: " + locyd);
          //}
          //Diffuse the neighborhood
          for (int i = dxLo; i < dxHi; i++) {
            for (int j = dyLo; j < dyHi; j++) {
              locnbID = i * resolution + j;
              if (pp == 0 || pp == 2 || pp == 4 || pp == 6 || pp == 8) {
                pp++;
                continue;
              }
              //choose only the Von Neumann's neighborhood
              else {
                //if(locxd==100 && locyd==100){
                //  System.out.println("loc Nb:x "+i+" x: "+j);
                //}
                cellnum = agentTorus.getCellSizeAt(i, j); //number of cells at that grid pt
                if(cellnum>0){
                	foodin += kFoodiff * 1 / (1 + cellnum) * (oldfoodLevel[locnbID] -
                    oldfoodLevel[locID]);}
                else{
                	foodin += kFoodiff * (oldfoodLevel[locnbID] - oldfoodLevel[locID]);}
                autocrin += kAutodiff * 1 / (1 + cellnum) *
                    (oldAutocrFood[locnbID] -
                     oldAutocrFood[locID]);
                ncells += agentTorus.getCellSizeAt(i, j);
                avgnbGluc += oldfoodLevel[locnbID];
                avgnbTGFa += oldAutocrFood[locnbID];
                //  if(oldfoodLevel[locID]<=0 && foodin >0){
                //    System.out.println("Catch food Infl > old level "+foodin);
                //  }
                pp++;
              }
            }
          }
          //the diffusion aglorithm for Glucose
          //       diffusFood[locID] = oldfoodLevel[locID]
          //         + (avgnbGluc/4 - oldfoodLevel[locID]) * kFoodiff * 1/(1+ncells);
          //the diffusion aglorithm for TGFa
          //     diffAutocr[locID] = oldAutocrFood[locID]
          //    + (avgnbTGFa/4 - oldAutocrFood[locID]) * kAutodiff * 1/(1+ncells);
//***** THIS CODE IS BEING TESTED
           diffusFood[locID] = oldfoodLevel[locID] + foodin;
          diffAutocr[locID] = oldAutocrFood[locID] + autocrin;
          //////The mass balance algorithm for diffuson
          /*foodout = 4*kFoodiff * 1/(1+cellLoc) * oldfoodLevel[locID];
               autocrout = 4*kAutodiff * 1/(1+cellLoc) * oldAutocrFood[locID];
                      diffusFood[locID] = foodin - foodout;
                      diffAutocr[locID] = autocrin - autocrout;
           */
//**** TEST OVER
           //if the diffusion makes the values negative, it is averaged
          if (diffusFood[locID] < 0) { //incase the level goes below 0
            diffusFood[locID] = avgnbGluc / 4;
          }
          if (diffAutocr[locID] < 0) {
            diffAutocr[locID] = avgnbTGFa / 4;
          }
          //else  diffusFood[locID] = oldfoodLevel[locID];
        }
        else {
          diffusFood[locID] = 0.0; //if at the boundaries
        }
        diffAutocr[locID] = 0.0; //dirichlet- sink at the boundaries

        if (diffusFood[locID] > foodLevelMax[locID]) {
          diffusFood[locID] = foodLevelMax[locID];
        }
        // if(diffusFood[locID] < 0){
        //     diffusFood[locID] = 0;
        //   }

      } //end of grid x
    } //end of grid y

    foodLevel = diffusFood; //UPDATE the foodLevel to the diffused level for that iteration
  } // end of Method to diffuse food

  public void updateAutocr() {
    int locID = 0;
    for (int ii = 0; ii < resolution; ii++) {
      int i = agentTorus.xnorm(ii);
      for (int jj = 0; jj < resolution; jj++) {
        int j = agentTorus.xnorm(jj);
        locID = i * resolution + j; // Retrieve the ID of this location
        if ( (autocrFood[locID] * rTgfexDeg) <= autocrFood[locID]) {
          autocrFood[locID] -= autocrFood[locID] * rTgfexDeg;
        }
      }
    }
    oldAutocrFood = autocrFood; //update autocr food
  }

// The method that marks which agents are in the core and which are at the edge
// and stores them in array lists for further use
  public void coredgMark() {
    int numMig = 0, numPro = 0; //number of migrating & proliferating cells
    coreList = new ArrayList(); // Create a new instance of list of proliferating agents
    edgeList = new ArrayList();
    //System.out.println("size of empty ArrayList "+ edgeList.size());
    double coreTGFrna = 0,
        edgeTGFrna = 0,
        coreEGFRrna = 0,
        edgeEGFRrna = 0,
        coreEGFRs = 0,
        edgeEGFRs = 0;
    double migTGFrna = 0, //TGF_rna- in each compartment- migrating cell
        migEGFRrna = 0,
        proTGFrna = 0,
        proEGFRrna = 0;

    Iterator it = agentList.iterator();
    while (it.hasNext()) {
      Agent a = (Agent) it.next();
      int nfreenb = noteEmptyNeighbor(a.x, a.y);
      //Dimension surfxy = getEmptyNeighbor(a.x, a.y);
      if (nfreenb == 0) { //if it is surrounded by other agents
        a.isCoreAg = true;
        coreList.add(a); // add an agent to the core list
        //calculate the TGFrna and EGFRrna total values in
        // all the cells of the core
        for (int cmpt = 0; cmpt < 4; cmpt++) {
          coreTGFrna += a.Xmol_new[cmpt][7];
          coreEGFRrna += a.Xmol_new[cmpt][8];
          coreEGFRs += a.Xmol_new[cmpt][1]
              + a.Xmol_new[cmpt][2]
              + a.Xmol_new[cmpt][3];
        }
      }
      else if (nfreenb > 0) { //if it has atleast 1 free neighbour
        a.isEdgeAg = true; //label agent as belonging to the edge
        edgeList.add(a); //add an agent to the edge list
        //calculate the TGFrna and EGFRrna total values in
        // all the cells of the edge
        for (int cmpt = 0; cmpt < 4; cmpt++) {
          edgeTGFrna += a.Xmol_new[cmpt][7];
          edgeEGFRrna += a.Xmol_new[cmpt][8];
          edgeEGFRs += a.Xmol_new[cmpt][1] //egfr_s free
              + a.Xmol_new[cmpt][2] //tgfa-egfr_s
              + a.Xmol_new[cmpt][3]; //pp-tgfa-egfr_s
        }
      }

      /////////////Evaluate Migratory and Proliferative cell Molec. Singnatures
      if (a.isMig) {
        for (int cnum = 0; cnum < 4; cnum++) {
          migTGFrna += a.Xmol_new[cnum][7];
          migEGFRrna += a.Xmol_new[cnum][8];
          migTGFaex[cnum] += a.Xmol_new[cnum][0]; //TGFa ex protein- migrating cell
          migPpLR[cnum] += a.Xmol_new[cnum][3]; // pp TGFa-EGFR_s protein- migrating cell
          migPLCg[cnum] += a.Xmol_new[cnum][9]; // PLCgamma protein - migrating cell
        }
        numMig++;
      }
      if (a.isProlif) {
        for (int cnum = 0; cnum < 4; cnum++) {
          proTGFrna += a.Xmol_new[cnum][7];
          proEGFRrna += a.Xmol_new[cnum][8];
          proTGFaex[cnum] += a.Xmol_new[cnum][0]; //TGFa ex protein- migrating cell
          proPpLR[cnum] += a.Xmol_new[cnum][3]; // pp TGFa-EGFR_s protein- migrating cell
          proPLCg[cnum] += a.Xmol_new[cnum][9]; // PLCgamma protein - migrating cell
        }
        numPro++;
      }
      //////////////

    }
    //average values (per cell)
    //calculate average molecular concentrations for core and edge
    if (coreList.size() > 0) { //to avoid division by ZERO
      coreTGFrna = coreTGFrna / coreList.size();
      coreEGFRrna = coreEGFRrna / coreList.size();
      coreEGFRs = coreEGFRs / coreList.size();
    }
    else if (coreList.size() == 0) {
      coreTGFrna = 0.0;
      coreEGFRrna = 0.0;
      coreEGFRs = 0.0;
    }
//calculate average molecular concentrations for core and edge
    if (edgeList.size() > 0) {
      edgeTGFrna = edgeTGFrna / edgeList.size();
      edgeEGFRrna = edgeEGFRrna / edgeList.size();
      edgeEGFRs = edgeEGFRs / edgeList.size();
    }
    else if (edgeList.size() == 0) {
      edgeTGFrna = 0.0;
      edgeEGFRrna = 0.0;
      edgeEGFRs = 0.0;
    }

// Average values of molecular profile Migration vs Proliferation
    if (numMig > 0) { //to prevent division by zero
      migTGFrna = migTGFrna / numMig;
      migEGFRrna = migEGFRrna / numMig;
    }
    if (numPro > 0) {
      proTGFrna = proTGFrna / numPro;
      proEGFRrna = proEGFRrna / numPro;
    }
    //   System.out.println("Number Prolfierating "+numPro +" num Migrating "+ numMig
    //                        +" migTGFrna " + migTGFrna+" proTGFrna " + proTGFrna);

  }

//the foll. methods allow access for the TumorModelBatch to model data types
  public double getmigTGFrna() {
    double migTGFrna_av = migTGFrna;
    return migTGFrna_av;
  }

  public ArrayList getCoreList() {
    return coreList;
  }

  public ArrayList getEdgeList() {
    return edgeList;
  }

//count the number of cells that neither migrate nor proliferate
  public void countMigProlif() {
    //switch parameter int[3]
    //0 = from proliferative (P=1) to migratory (M=2) states
    //1 = from migratory (M=2) to proliferative (P=1) states
    //2 = from quiescent (Q=0) to pro (P=1) or mig (M=2) states
    numSwitch = new int[4]; //this array contains values of cells that switch
    totnSwitch = 0;//keeping track of cells changing phenotypes
    totAg = 0;

    prolifList = new ArrayList();
    migList = new ArrayList();
    quiescList = new ArrayList();
    Iterator Pit = agentList.iterator();



    while (Pit.hasNext()) {
      Agent aP = (Agent) Pit.next();
      if (aP.isQuiesc) {
        quiescList.add(aP);
        aP.phenoArr[1] = 0;
      }
      else if (aP.isProlif) {
        prolifList.add(aP);
        aP.phenoArr[1] = 1;
      }
      else if (aP.isMig) {
        migList.add(aP);
        aP.phenoArr[1] = 2;
      }
      //if there is a CHANGE Mig-->Pro
      if (aP.phenoArr[0] != aP.phenoArr[1] && aP.phenoArr[0] == 1 &&
          aP.phenoArr[1] == 2) {
        //System.out.println("state of old Phenotype: " + aP.phenoArr[0] +
        //                   " state of new Phenotype: " + aP.phenoArr[1]);
        numSwitch[0] += 1;
      }
      //if there is a CHANGE Pro-->Mig
      else if (aP.phenoArr[0] != aP.phenoArr[1] && aP.phenoArr[0] == 2 &&
               aP.phenoArr[1] == 1) {
        //System.out.println("state of old Phenotype: " + aP.phenoArr[0] +
        //                   " state of new Phenotype: " + aP.phenoArr[1]);
        numSwitch[1] += 1;
      }
      //if there is a CHANGE Quies-->Mig/Pro
      else if (aP.phenoArr[0] != aP.phenoArr[1] && aP.phenoArr[0] == 0) {
        //System.out.println("state of old Phenotype: " + aP.phenoArr[0] +
        //                   " state of new Phenotype: " + aP.phenoArr[1]);
        numSwitch[2] += 1;
      }
      //if there is a CHANGE IN PHENOTYPE before and after a RUN
      else if (aP.phenoArr[0] == aP.phenoArr[1]) {
        numSwitch[3] += 1;
      }

      if(aP.phenoArr[0] != aP.phenoArr[1]){
        totnSwitch +=1;
      }

      // TO DETECT THE MOLECULAR CAUSE of phenotypic switching, between RUNS
      // beyond the obvious PLCgamma change of conc., upstream or downstream in the network
     /*  if (aP.phenoArr[0] != aP.phenoArr[1]) {
        System.out.println(getTickCount() + ", " + aP.agentID + ", " +
                           aP.phenoArr[0] + ", " +
                           aP.phenoArr[1] + ", ");

        for (int compn = 0; compn < 4; compn++) {

          System.out.println(aP.Xmol_new[compn][0] - aP.Xmol_ia[compn][0]);
           System.out.println(aP.Xmol_new[compn][0] + ", " + //TGF_ex
                           aP.Xmol_new[compn][3] + ", " //pp-TGFa-EGFRs
                           + aP.Xmol_new[compn][1]+ ", "//EGFRs
                           + aP.Xmol_new[compn][9] + ", "//p-PLCg
                           + aP.Xmol_new[compn][7] + ", "//TGF-rna
                           + aP.Xmol_new[compn][8]); //EGFR-rna

        }

      }*/
     totAg +=1;
    }

    System.out.println(getTickCount() + ", " +
                       //" Mig#: " + migList.size() +
                       //" Pro#: " + prolifList.size() +
                       //" P2M: " +
                       numSwitch[0] + ", " +
                       //" M2P: " +
                       numSwitch[1] + ", " +
                       //" Q2mp: " +
                       numSwitch[2] + ", " +
                       //" NoCh: " +
                       numSwitch[3]+ ", " +
                       //total number that change phenotype
                       totnSwitch);
                       // + ", " +totAg);//TOTAL NUM AGENTS- debug
  }

  public ArrayList getProlifList() {
    return prolifList;
  }

  public ArrayList getMigList() {
    return migList;
  }

///////////
  //Box counting for how many cells fall within a given box size
  //
  public void fractalBoxCount() {
    int xini = 1, xend = resolution, yini = 1, yend = resolution, bsize1 = 2,
        bsize = 0, xstart = 1, ystart = 1, nboxes = 0, boxincr = 1, numinit = 0;
    int maxbox = ( (resolution - bsize1) ^ 2) / (4 * boxincr); //1/4th of the full length
    //the max extent of box sizes dep on: init box siz, box increm, and 25% length of dataset
    boxList = new double[2][maxbox]; //[((resolution - bsize1)^2) / boxincr]; //java integer division always gives integers

//changing the box size
    for (int n = 0; n < maxbox - 1; n++) {
      bsize = n + bsize1;
      //  int[][] boxcount;
      int numcel = 0;
      //if the previous count of boxes is =1, stop changing box size
      if (n > 0 && (int) boxList[1][n - 1] == 1) {
        break;
      }

      nboxes = (resolution / bsize);

      // System.out.println("box size: " + bsize+" num boxes: "+nboxes);
      //int[] boxList;
      //vary the start point of the boxes
      //numcel = 0; //reset the value of the number of cells counted for a box-size
      //move the initial box
      numinit = 0;
      for (int m = 0; m < (bsize - 1); m++) {
        xstart = xini + m;
        //  for (ystart = yini; ystart < (yini + bsize - 1); ystart++) {
        //for a given initial position of box, move the box over the grid
        for (int xbox = xstart; xbox < xend; xbox += bsize) {
          for (int ybox = ystart; ybox < yend; ybox += bsize) {

            //count the number of cells in the
            boolean boxIscount = false;
            //move through the grid points of the box
            for (int xst = xbox; xst < (xbox + bsize); xst++) {
              for (int yst = ybox; yst < (ybox + bsize); yst++) {
                //detects if any of the grid points in the box have cells
                if (agentTorus.getCellSizeAt(xst, yst) > 0) {
                  boxIscount = true;
                }
              }
            }
//
            if (boxIscount == true) {
              // boxList[numList] = 1;
              numcel += 1;
              //boxList[
            }
            // System.out.println("x " + xbox + " y " + ybox +
            //                   "No. of Cells at a locn: " + numcel);

            //}
          }
        }
        numinit += 1;
      }
      //average the count of the boxes for a given box size
      numcel = numcel / numinit;
      boxList[1][n] = numcel;
      boxList[0][n] = bsize;
//      System.out.println("Boxsize: " +boxList[0][n] + ", Boxcount: " + boxList[1][n]);
      //numList++;
    }
    int listlen = boxList[0].length;
    //  System.out.println("length: " + listlen);
    int nonzer = 0;
    //In order to identify the lengh of the array that still contains data
    for (int intq = 0; intq < listlen - 1; intq++) {
      if (boxList[0][intq] > 0) {
        nonzer = intq;
        // System.out.println("q: "+intq);
      }
      else {
        break;
      }
    }
    nonzer += 1;
    //System.out.println(" length of nonzero arr: "+nonzer);
    // call the linear least square fitting routine/method
    //System.out.println("@boxcount length array (row): "+boxList[0].length);
    double[] linefit = linFit(boxList, nonzer);
//    System.out.println("@fractalCt, b: "+linefit[0]+ " a: "+linefit[1]+
//                     " r^2:" +linefit[2]);
  }

  //////////

  ////////LINEAR LEAST SQUARE FITTING ROUTINE
  //Function: y= a + bx
  public double[] linFit(double[][] xyarr, int nzlen) {

    xydata = new double[2][nzlen]; //array with the x- and y- data
//    fitvals = new double[3];//Fitvals=[ a, b, and r^2]
    double xbar = 0.0, ybar = 0.0, a_x = 0.0, b_x = 0.0, r_sq = 0.0,
        sxx = 0.0, syy = 0.0, sxy = 0.0, temp_x = 0.0, temp_y = 0.0;

    //the data is now copied to an array with the correct length
    for (int r = 0; r < nzlen; r++) {
      xydata[0][r] = Math.log(xyarr[0][r]);
      xydata[1][r] = Math.log(xyarr[1][r]);
      xbar += xydata[0][r];
      ybar += xydata[1][r];
    }
    //System.out.println("xav: "+xbar+" yav: "+ ybar);
    //Averages of the x- and y-series
    xbar = xbar / nzlen;
    ybar = ybar / nzlen;

    for (int ss = 0; ss < nzlen; ss++) {
      sxx += Math.pow( ( (double) xydata[0][ss] - xbar), 2);
      syy += Math.pow( ( (double) xydata[1][ss] - ybar), 2);
      sxy += ( (double) xydata[0][ss] - xbar) * ( (double) xydata[1][ss] - ybar);
    }
    //System.out.println("sxx: "+sxx+" syy: "+ syy+" sxy: "+sxy);
    b_x = sxy / sxx;
    a_x = ybar - (b_x * xbar);
    r_sq = Math.pow(sxy / (Math.sqrt(sxx * syy)), 2);
//    System.out.println("b: "+b_x+" a: "+ a_x+" r^2: "+r_sq);
    fitvals[0] = b_x;
    fitvals[1] = a_x;
    fitvals[2] = r_sq;
    return fitvals;
  }

  //////

////////////////////Is the Neighborfree? then Note it.////////
  // This method notes which torus locations in the Moores Neighborhood of
  // the agent are free.
  // noteEmptyNeighbour(); //Marking neighbouring cells that are free for prolif or migrat.
// Array list containing free locations of Dimension class
// in the neighborhood of an agent
//ArrayList emptyneighborList;
  // Find out if there are any empty neighbours:if YEs: answer >0 ELSE ==0
  public int noteEmptyNeighbor(int x, int y) {
    ArrayList freePosNg = new ArrayList();
    //System.out.println("Size of empty Array: " + freePosNg.size());
    int freeposID = 0;
    int vnnb = 0;

    for (int l = xLowerBound[x]; l < xUpperBound[x]; l++) {
      int ll = agentTorus.xnorm(l); //xnorm
      for (int m = yLowerBound[y]; m < yUpperBound[y]; m++) {
        int mm = agentTorus.xnorm(m); //ynorm
        if (vnnb == 0 || vnnb == 2 || vnnb == 4 || vnnb == 6 || vnnb == 8) {
          //System.out.println("compt avoided: "+ vnnb);
          vnnb++;
          continue;
        }
        else {
          if (agentTorus.getCellSizeAt(ll, mm) == 0) {
            int locID = ll * resolution + mm;
            freePosNg.add(new Dimension(ll, mm));
            freeposID++;
          }
          //System.out.println("compt considered: "+ vnnb);
          vnnb++;
        }
      }
    } //VNN neihgborhoods
    //System.out.println("Num free pos: "+freeposID+" size of free pos arr: "+
    //                   freePosNg.size());
    return freePosNg.size();
  }

  // Get the coordinates of an empty neighbour
  public Dimension getEmptyNeighbor(int x, int y) {
    ArrayList freePosNg = new ArrayList();
    //System.out.println("Size of empty Array: " + freePosNg.size());
    int freeposID = 0;
    int vnn = 0; //von neumann neighborhood (vnn)

    //for the moores neighborhood
    for (int l = xLowerBound[x]; l < xUpperBound[x]; l++) {
      int ll = agentTorus.xnorm(l); //xnorm
      for (int m = yLowerBound[y]; m < yUpperBound[y]; m++) {
        int mm = agentTorus.xnorm(m); //ynorm
        if (vnn == 0 || vnn == 2 || vnn == 4 || vnn == 6 || vnn == 8) {
          vnn++; //increment the counter
          continue; //skip the non-VNN grid pts
        }
        else {
          if (agentTorus.getCellSizeAt(ll, mm) == 0) {
            //int locID = ll * resolution + mm;
            freePosNg.add(new Dimension(ll, mm));
            freeposID++;
          }
          vnn++;
        }
      }
    } //The VNN nbhood

    //  System.out.println("Num free pos: "+freeposID+" size of free pos arr: "+
    //                     freePosNg.size());

    Dimension freePosit2D = new Dimension();
    if (freePosNg.size() > 0) { //values 1 to 4
      int positrand = getNextIntFromTo(0, freePosNg.size() - 1); //here we get the new free
      //location in the list of empty neighbors based on a uniform random number
      //between the size of the old array and the updated one.
      freePosit2D = (Dimension) freePosNg.get(positrand);
    }
    else
    if (freePosNg.size() == 0) {
      freePosit2D = new Dimension();
    }
    return freePosit2D;
  }

////////////

  // The method that mark which agents will proliferate (but not create new agents)
  public void proliferate() {

    proliferateList = new ArrayList(); // Create a new instance of list of proliferating agents
    Iterator it = agentList.iterator();
    while (it.hasNext()) {
      Agent a = (Agent) it.next();

      if (!a.isProlifSet) { // If proliferation not decided yet for this agent
        a.isProlifSet = true; // then set the flag as "true"
        int surf = noteEmptyNeighbor(a.x, a.y);
        // System.out.println("nEmpty nbs: "+ surf);
        double[] pProlif = {
            0.0, 0.0}; //the array with PLCg and pLR conc
        // Chaitanya: this should be an or "||" condition, not and "&&"
        //if (surf > 0) {
        double totpLRs = 0;
        // Total compartmental values of the molecules
        // governing proliferation choice is calculated (totpLR)
        for (int compno = 0; compno < totalCompartments - 1; compno++) {
          totpLRs += a.Xmol_new[compno][3];
        }
        //IF THE DECISION TO MIGRATE IS BASED ON THE GRADIENT OF PLCg
        pProlif[0] = a.maxGradPLC;
        //System.out.println("maxPLC Grad"+ pProlif[0]);
        //IF MIGRATION DECISION IS TAKEN BASED ON ABSOLUTE CONC of PLCg
        //pProlif[0] = a.maxPLC;
        //System.out.println("pProlif_1" + pProlif[0]);
        pProlif[1] = totpLRs / 4; //Equate the pProlif to molec concentrations

        updateProlifList(a, pProlif); // Update the prolifList
        // System.out.println("Prolif updated");
        //}
        //else
        //  System.out.println("surrounded");
      }
    }
//       System.out.println("At proliferate(),t=" + getTickCount() +
//                          ",numAgents=" + agentList.size());
  }

  // The method that updates the proliferateList
  public void updateProlifList(Agent a, double[] pBirth) {
    //double p = 0.0;
    int locID = a.x * resolution + a.y;
    int freesurf = noteEmptyNeighbor(a.x, a.y);
    //CHANGED# proliferation occurs in response to
    //birth pProlif exceeding a threshold
    //   if (freesurf > 0) {
    if (pBirth[0] >= threshPLC && freesurf > 0) {
      migrCell[locID] += 1;
      a.migLag += 1; //DELAY for migration
      if (a.migLag > threshLagMig - 1) {
        a.isMig = true; //the cell migrates
        a.isProlif = false; //DOESNT proliferate
        a.isQuiesc = false; //DOESNT proliferate
      }
    }
    else if (pBirth[0] < threshPLC && pBirth[1] >= threshpLRprol &&
             freesurf > 0) {
      a.isProlif = true;
      a.isMig = false;
      a.isQuiesc = false;
      proCell[locID] += 1; //add to the list of proliferating cells
      a.prolifLag += 1;
      if (a.prolifLag > threshLagPro - 1) {
        proliferateList.add(a);
        //        System.out.println("IS Proliferate");
      }
    }
    else {
      a.isProlif = false;
      a.isMig = false;
      a.isQuiesc = true;
      quiescCell[locID] += 1;
    }
    //  }

    //      System.out.println("IS Quiesce");
    //the cells sit quiety (senesce)

  }

////////////////////////// METHOD TO TRACK MOLEC VALS IN A SINGLE CELL
  public void getMolvals() {
    int numproCell = 0;
    int nummigCell = 0;
    double maxav_m = 0, maxav_p = 0;

    Iterator ita = agentList.iterator(); //to get all agents
    while (ita.hasNext()) {
      Agent a = (Agent) ita.next();
      // Get the molecular profile of the first cell in the peak zone

      if (avgsingle == 0) {
        if (a.agentID == 1668) {
          //System.out.println("Time" + getTickCount());
          if (printallmol == 1) {
            //System.out.println("The first cell: Coords X,"
            //                 + a.x + ", Y, " + a.y +
            //                 " agentID: " + a.agentID);
            /*
                         for (int compn = 0; compn < 4; compn++) {
              System.out.println("compn, " + compn +
                                 // ", TGFa-ex, " + a.Xmol_new[compn][0] +
                                 // ", pp-LR_s, " + a.Xmol_new[compn][3] +
                                 ", PLCg-act, " + a.Xmol_new[compn][9] +
                                 // ", EGFR-tot, "
                                 // + (a.Xmol_new[compn][1] +
                                 //    a.Xmol_new[compn][2] +
                                 //    a.Xmol_new[compn][3]) +
                                 ", TGFa-rna, " + a.Xmol_new[compn][7] +
                                 ", EGFR-rna, " + a.Xmol_new[compn][8]);
                         }
             */
          }
        }
        ///////////
      }
      else if (avgsingle == 1) {
        //output averaged values of molecules in each subcompartment
        //All cells in the 3rd Quadrant
        if (a.x >= resolution / 2 && a.y <= resolution / 2) {
          //System.out.println("Time" + getTickCount());
          //if (printallmol == 1) {
          //All proliferating cells
          if (a.isProlif) {
            for (int compn = 0; compn < 4; compn++) {
              proCellav[compn] = +a.Xmol_new[compn][9];

            }
            numproCell++;
          }
          // if (!a.isProlif && !a.isQuiesc) {
          if (a.isMig) {
            for (int compn = 0; compn < 4; compn++) {
              migCellav[compn] = +a.Xmol_new[compn][9];
              //System.out.println(a.Xmol_new[compn][9]);

            }
            nummigCell++;
          }

          //}
          //}
          //types of cells
        }
      }
    }
    for (int compn = 0; compn < 4; compn++) {
      proCellav[compn] = proCellav[compn] / (numproCell + 1);
      migCellav[compn] = migCellav[compn] / (nummigCell + 1);
      maxav_p = (maxav_p < proCellav[compn]) ? proCellav[compn] : maxav_p;
      maxav_m = (maxav_m < migCellav[compn]) ? migCellav[compn] : maxav_m;
    }

    //if (maxav_p > 0 || maxav_m > 0) {
    if (avgsingle == 1) {
      //System.out.println(
      //"Time, " + getTickCount()
      //  "Pro, " + proCellav[0] + ", Mig, " + migCellav[0] + "\n"
      //  + "Pro, " + proCellav[1] + ", Mig, " + migCellav[1] + "\n"
      //  + "Pro, " + proCellav[2] + ", Mig, " + migCellav[2] + "\n"
      //  + "Pro, " + proCellav[3] + ", Mig, " + migCellav[3]);
    }

    //}

  }

  //////////////////// RANDOM WALK OF CELLS

  // to measure the path of the migrating cells closest to the thrid quadrant
  public void compPath() {
    Iterator itm = agentList.iterator(); //to get all agents
    //System.out.println("Time, " + getTickCount());
    int numLastcells = 0;
    while (itm.hasNext()) {
      Agent a = (Agent) itm.next();
      //If the cells are in the 3rd quadrant and migrating
      if (a.isMig && a.x >= resolution / 2 && a.y <= resolution / 2) {
        //if (a.isMig && a.dist3Quad() < 25) {
        migCellPLC = new double[4]; //initialize it for each cell
        //if(a.distEucl3Quad() < 15 && a.isMig){

        /*for (int compn = 0; compn < 4; compn++) {
          migCellPLC[compn] = +a.Xmol_new[compn][9];
                 }*/
        System.out.println(getTickCount() + ", " + a.agentID + " , "
                           + a.x + ", " + a.y);
        numLastcells += 1;
      }
    }
    /*if (numLastcells > 0) {
      for (int compn = 0; compn < 4; compn++) {
        migCellPLC[compn] = migCellPLC[compn] / numLastcells;
      }
      System.out.println("Avg Last migcell PLC, " + migCellPLC[0] + ", "
                         + migCellPLC[1] + ", "
                         + migCellPLC[2] + ", "
                         + migCellPLC[3] + ", ");
         }*/

  }

////////////// the Method to calculate the mean and standard deviation for a 2D
  //distribution of the tumor
  public void surfRough() {
    Iterator itag = agentList.iterator(); //to get all agents
    double distorig = 0.0;
    double stdevDist = 0.0;
    double Avgdist = 0.0;
    double[] distcent = new double[agentList.size()];
//       System.out.println("num Agents: " + agentList.size());
    int agentnum = 0;
    int nbhood = 0;
    int[] origin = {
        resolution / 2, resolution / 2};
    while (itag.hasNext()) {
      Agent a1 = (Agent) itag.next();

      //////////////EDGE OR CORE?
      nbhood = noteEmptyNeighbor(a1.x, a1.y);
      //System.out.println("num of free nbrs" + nbhood);

      if (nbhood > 0) {
        int[] agpos = {
            a1.x, a1.y};
        distorig += computeEuclD(origin, agpos); //distance to centre
        distcent[agentnum] = distorig;
        agentnum++; //number of edge agents
      }
    }
    //end of agentList
    Avgdist = distorig / agentnum;
    //calculate square difference from avg
    for (int n = 0; n < agentnum; n++) {
      StdevDist += Math.pow( (distcent[n] - Avgdist), 2.0);
    }
    //average the difference
    StdevDist = (Math.pow(StdevDist, (1 / 2))) / agentnum;

    Roughness[0] = Avgdist;
    Roughness[1] = StdevDist;
    // System.out.println("total ag #: " + agentnum + ", avg dist " +
    //                    Roughness[0] + ", stdev: " + Roughness[1]);

  }

//////////////////////////////////

  public void calcMigProRatio() {
    Timept = getTickCount();
    Migpro = Agent.alreadyMoved / (1 + getProlifList().size()); //
    //System.out.println("Time: " + getTickCount());
    //", Ratio: "+Migpro );
    //RatioMigpro[(int)Timept]=Migpro;
  }

////////////////////////////////////

  /* public void storeTraj(){
     Iterator iter = agentList.iterator(); //to get all agents
     while (iter.hasNext()) {
       Agent ag = (Agent) iter.next();
       agTrack[ag.agentID][0]=(double) getTickCount();
       agTrack[ag.agentID][1]=ag.agentID;
       agTrack[ag.agentID][2]=ag.x;
       agTrack[ag.agentID][3]=ag.y;
     }
   }*/

//calculating the inter-agent distance at each time pt.
  public double avDistAg() {
    double avAgdist = 0.0;
    xyAgents = new int[agentList.size()][2];
    int numMigAg3q = 0; //number of migrating agents at each iteration
    Iterator itm = agentList.iterator(); //to get all agents

    while (itm.hasNext()) {
      Agent agm = (Agent) itm.next();
      if (agm.isMig && agm.x >= resolution / 2 && agm.y <= resolution / 2) {
        xyAgents[numMigAg3q][0] = agm.x;
        xyAgents[numMigAg3q][1] = agm.y;
        numMigAg3q++; //number of migrating agents
        //System.out.println("@Avgdist x="+agm.x+", y="+agm.y);
      }
    }
    if (numMigAg3q > 0) { //if there are any migrating cells at a given time pt.
      for (int n = 0; n < numMigAg3q; n++) {
        int[] pt1 = xyAgents[n]; //each agent's location
        for (int m = 0; m < numMigAg3q; m++) {
          int[] pt2 = xyAgents[m]; //{0,0};//the corresponding neighbour
          avAgdist += (double) computeEuclD(pt1, pt2);
        }
      }
      if (numMigAg3q > 0) {
        avAgdist = avAgdist / (double) (numMigAg3q * numMigAg3q);
        //ensure NO DIVISION BY ZERO
      }
      //System.out.println(getTickCount() + ", " +
      //                 numMigAg3q + ", " +
      //                avAgdist);
    }
    else {
      //System.out.println(getTickCount() + ", " +
      //                 numMigAg3q + ", " + avAgdist);

    }
    return avAgdist;
  }

// The method that returns agentList, since this object is protected
  public ArrayList getAgentList() {
    return agentList;
  }

// The method that returns list of Proliferating, since this object is protected
  public ArrayList getproliferateList() {
    return proliferateList;
  }

// The step method
  public void step() {
    if (schedule.getCurrentTime() < 1) {
      return;
    }
    else {
      super.step();

    }
  }

// The postStep method
  public void postStep() {
    for (int j = 0; j < numLocations; j++) {
      migrCell[j] = 0; //empty the migrCells[]
      quiescCell[j] = 0; //Empty the quiesc cell array
      proCell[j] = 0; //Empty the array of proliferating cells
    }

    /*  simulation if quadrant 3 has been invaded, and record elapsed time */
    if (this.isThirdQuadrantReached) { // If third quadrant has been successfully invaded
      stop();

      surfRough(); //calculate the standard deviation of Edge agents from centre

      fractalBoxCount(); //perfom box counting on the last time step
      //Iterator ita = agentList.iterator();
      Iterator ita = agentList.iterator(); //to get only the outline
      while (ita.hasNext()) {
        Agent a = (Agent) ita.next();
        //System.out.println("Xcoord " + a.x + " Ycoord " + a.y);
        // Get the molecular profile of the first cell in the peak zone
        /*if (a.is3QFirst) {
          System.out.println("First in 3rd quadrant, X,"
                             + a.x + ", Y, " + a.y +
                             ", AgentID: " + a.agentID);
          if (printallmol == 1) {
            for (int compn = 0; compn < 4; compn++) {
              System.out.println("compn, " + compn +
                                 // ", TGFa-ex, " + a.Xmol_new[compn][0] +
                                 // ", pp-LR_s, " + a.Xmol_new[compn][3] +
                                 ", PLCg-act, " + a.Xmol_new[compn][9] +
                                 // ", EGFR-tot, "
                                 // + (a.Xmol_new[compn][1] +
                                 //    a.Xmol_new[compn][2] +
                                 //    a.Xmol_new[compn][3]) +
                                 ", TGFa-rna, " + a.Xmol_new[compn][7] +
                                 ", EGFR-rna, " + a.Xmol_new[compn][8]);
            }
          }
          if (a.distEucl3Quad() <= foodPeakRange + 10 && a.isMig) {
            System.out.println("Migr ID: " + a.agentID +
                               " Xcoord " + a.x +
                               " Ycoord " + a.y);
            for (int compn = 0; compn < 4; compn++) {
              System.out.println("compn, " + compn +
                                 ", TGFa-ex, " + a.Xmol_new[compn][0] +
                                 ", pp-LR_s, " + a.Xmol_new[compn][3] +
                                 ", PLCg-act, " + a.Xmol_new[compn][9] +
                                 ", EGFR-tot, "
                                 + (a.Xmol_new[compn][1] +
                                    a.Xmol_new[compn][2] +
                                    a.Xmol_new[compn][3]) +
                                 ", TGFa-rna, " + a.Xmol_new[compn][7] +
                                 ", EGFR-rna, " + a.Xmol_new[compn][8]);
            }
                     }
                 }*/

        //System.out.println("Other cells near source");
        /*        if(a.distEucl3Quad() <= foodPeakRange + 5){
                  System.out.println("X,"
                                     + a.x + ", Y, " + a.y);
                  for (int compn = 0; compn < 4; compn++) {
                    System.out.println("compn, " + compn +
                                       ", TGFa-ex, " + a.Xmol_new[compn][0] +
                                       ", pp-LR_s, " + a.Xmol_new[compn][3] +
                                       ", PLCg-act, " + a.Xmol_new[compn][9] +
                                       ", EGFR-tot, "
                                       + (a.Xmol_new[compn][1] +
                                          a.Xmol_new[compn][2] +
                                          a.Xmol_new[compn][3]) +
                                       ", TGFa-rna, " + a.Xmol_new[compn][7] +
                                       ", EGFR-rna, " + a.Xmol_new[compn][8]);
                  }
                }*/
      }

          /*         Iterator itm = migList.iterator();//to get only the migrating cells
          //System.out.println("Migrating Cells:");
          int migno =0;
                 while (itm.hasNext()) {
        Agent am = (Agent) itm.next();
        migno++;
        if(am.distEucl3Quad() <= 20){
          System.out.println("Migr ID: "+am.agentID+
       " Xcoord " + am.x +
       " Ycoord " + am.y);
           for (int compn = 0; compn < 4; compn++) {
             System.out.println("compn, " + compn +
       ", TGFa-ex, " + am.Xmol_new[compn][0] +
       ", pp-LR_s, " + am.Xmol_new[compn][3] +
       ", PLCg-act, " + am.Xmol_new[compn][9] +
          ", EGFR-tot, "
       + (am.Xmol_new[compn][1] +
       am.Xmol_new[compn][2] +
       am.Xmol_new[compn][3]) +
       ", TGFa-rna, " + am.Xmol_new[compn][7] +
       ", EGFR-rna, " + am.Xmol_new[compn][8]);
           }
        }
                 }
                 System.out.println("number of mig= "+ migno);
           Iterator itp = prolifList.iterator();//to get only the prolif cells
          System.out.println("Proliferating Cells:");
         while (itp.hasNext()) {
           Agent ap = (Agent) itp.next();
       System.out.println("Xcoord " + ap.x + " Ycoord " + ap.y);
         }
       */

      long endTime = System.currentTimeMillis(); // Record end time
      long elapsedTime = endTime - startTime;
      elapsedTimeSecond = (double) elapsedTime / 1000.0;
      System.out.println("At postStep of Network Model #@#," +
                         elapsedTimeSecond +
                         " seconds elapsed for this run");

    }
//NOTE: PLEASE REMOVE THIS IF LOOP AFTER TESTING THE CODE
//IT restricts the simulation to only 10 time steps.
    else if (getTickCount() == 1) {
      fractalBoxCount(); //perfom box counting on the first time step
    }
    /*   else if (getTickCount() == 100) {
          long endTime = System.currentTimeMillis();
         long elapsedTime = endTime - startTime;
         elapsedTimeSecond = (double) elapsedTime / 1000.0;
         System.out.println("At Post step of NETWORK simulation (EXCEEDED MAX ITERAT num), " +
                            elapsedTimeSecond +
                            " seconds elapsed for this run of iteration= " +
                            getTickCount());
         stop();
         Iterator ita = agentList.iterator();
                     while (ita.hasNext()) {
                       Agent a = (Agent) ita.next();
                       System.out.println("Xcoord "+ a.x+" Ycoord "+a.y);
           }
           int locID = 0;
            for (int ii = 0; ii < resolution; ii++) {
            int i = agentTorus.xnorm(ii);
              for (int jj = 0; jj < resolution; jj++) {
              int j = agentTorus.xnorm(jj);
                locID = i * resolution + j; // Retrieve the ID of this location
                if(foodLevel[locID] > 0.00001){
            //      System.out.println("X, "+ i + ", Y, " + j + ", food, " +
            //                         foodLevel[locID]);
                }
              }
            }
       }
     */
///////////////////////////////////////////////////////////
    moveAgent(); // Move migrating agents to their new locations
    generateNewAgents(); // Call the method that generates new agents

    calcMigProRatio(); //get the migration/proliferation ratio
    //  if(getTickCount() > 200){getMolvals();}
    //CALL THE METHOD TO TRACK SINGLE CELLS

//    compPath(); // call the method to track coordinates of cells

    //avDistAg(); //calculate avg inter-agent distance for each time pt
    //  }
//   surfRough(); //calculate the standard deviation of Edge agents from centre

//    calcMolMigPro();//calculate Molec. profiles of mig and pro cells
    coredgMark(); // Marking which cells are at the edge and which are at the core
    countMigProlif(); //count number of proliferating agents

    /*    if (getTickCount() == 10) {
//      fractalBoxCount(); //count the boxes of a given size
          double[][] testfit={{10, 20, 30, 40},//x
              {1,10,100,1000}}; //y
          int lentest = 4;
          double[] testout = linFit(testfit,lentest);
          System.out.println("test: b "+testout[0]+" a "+testout[1]+" r^2"
                             +testout[2]);
        }
     */

  }

// The method that moves migrating agents to their new locations
  public void moveAgent() {
    Iterator it = movingList.iterator();
    while (it.hasNext()) {
      //System.out.println("Agents in moving list: " + movingList.size());
      Agent a = (Agent) it.next();
      //  if ( (agentTorus.getCellSizeAt(a.newX, a.newY) == 0)){
      //         && (a.newX > 0 || a.newY > 0)) {
      a.placeTo(a.newX, a.newY); // Now place the agent in the new location
      int migrLocID = a.newX * resolution + a.newY;
      //       if (showMigrCell) { // Are we showing migrating on the display
      //  migrCell[migrLocID]+=1;
// EITHER: MigrCell is updated here, OR at updateProliferate()
      // Chaitanya: we need to reset the migrCell array in every period
      // if (migrCell[migrLocID] > 0) {
      //to make sure at a given loc ID has only 1 cell
      //     System.out.println("Migr cell @locId is > 0! " +
      //                        migrCell[migrLocID]);

      //}
      //    }
      //else {
      //  a.placeTo(a.x, a.x);
      //}
      //System.out.println("no migration-freePOS");
    }

    //Chaitanya2self: Is a single migrating cell being added more than once to movingList??
    //System.out.println("how many migrate: " + movingList.size());
    movingList = null; // Reset the list of moving agents
    //migrCell = new int[numLocations];
  }

//////////////////////////////////////////////////////////////
// Here new agents are generated
///////////////////////////////////////
// The method that generates new agents
  public void generateNewAgents() {
    int numProliferate;
    int numNewAgents = 0;
    numProliferate = proliferateList.size();
    double t = getTickCount(); // Retrieve the current time-period
    // Now we compute the number of newly-born agents in this period
    // using Gompertz growth function
    numNewAgents = (int) (numProliferate * (Math.exp(1.0 - Math.exp( -t)) - 1.0));
    ArrayList birthList = new ArrayList(); // Create a list of new agents
    // The new agent is placed at one of the parents' location using an iterator
    // if the iteration finishes before all new agents are located, the iterator
    // is re-created until all new agents have been given a location
    Iterator it = proliferateList.iterator();
    if (!it.hasNext()) {
      return;
    }
    for (int i = 0; i < numNewAgents; i++) {

      if (!it.hasNext()) { // Check if the iterator is empty
        it = proliferateList.iterator();
      } // If empty, reset the iterator

      Agent oldAgent = (Agent) it.next(); // Get a "parent" agent
      int freeNb = noteEmptyNeighbor(oldAgent.x, oldAgent.y);

      if (freeNb > 0) { //only if the there is free space in the neighborhood

        Dimension nbLoc = getEmptyNeighbor(oldAgent.x, oldAgent.y);
        int newXcoord = nbLoc.width;
        int newYcoord = nbLoc.height;
        oldAgent.prolifLag = 0; //reset proliferation lag for old agent
        oldAgent.migLag = 0;

        double[][] Molecinit = {
            {
            0.0, 25.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.1,
            0.1, 0.0, 1.0, 0.0,
            1.0, 5.0, 0.0, 0.0}
            ,
            //compartment 0

            {
            0.0, 25.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.1,
            0.1, 0.0, 1.0, 0.0,
            1.0, 5.0, 0.0, 0.0}
            ,
            //compartment 1

            {
            0.0, 25.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.1,
            0.1, 0.0, 1.0, 0.0,
            1.0, 5.0, 0.0, 0.0}
            , //compartment 2

            {
            0.0, 25.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.1,
            0.1, 0.0, 1.0, 0.0,
            1.0, 5.0, 0.0, 0.0}
        };
        for (int cno = 0; cno < 4; cno++) {
          Molecinit[cno][1] = initEGFRs;
        }
        oldAgent.Xmol_ia = Molecinit;

        agentID++; // Increment the counter of agent ID
        Agent newAgent = new Agent(agentID, this); // Create a new agent
        birthList.add(newAgent); // Add the new agent into the temporary list
        newAgent.placeTo(newXcoord, newYcoord);
        newAgent.flagThirdQuadrant();

        double[][] reinit = {
            {
            0.0, 25.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.1,
            0.1, 0.0, 1.0, 0.0,
            1.0, 5.0, 0.0, 0.0}
            , //compartment 0

            {
            0.0, 25.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.1,
            0.1, 0.0, 1.0, 0.0,
            1.0, 5.0, 0.0, 0.0}
            , //compartment 1

            {
            0.0, 25.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.1,
            0.1, 0.0, 1.0, 0.0,
            1.0, 5.0, 0.0, 0.0}
            , //compartment 2

            {
            0.0, 25.0, 0.0, 0.0,
            0.0, 0.0, 0.0, 0.1,
            0.1, 0.0, 1.0, 0.0,
            1.0, 5.0, 0.0, 0.0}
        };
//compartment 3

        newAgent.Xmol_ia = reinit;
        newAgent.prolifLag = 0; //reset the proliferation counter for the daughter cell
        newAgent.migLag = 0;
        // Now add all newly-born agents to the agent list
      }
      else {
        // System.out.println("dont proliferate");
        continue;
      }
    }
    agentList.addAll(birthList);
    numNewAgent = birthList.size(); // Size of birth list

    proliferateList = null; // Empty arrayList
  }

/////////////////////////////////////////////////////////////////////////
// Creating and starting the tumor model  ///////////////////////////////
/////////////////////////////////////////////////////////////////////////
  public static void main(String[] args) {
    SimInit init = new SimInit();
    TumorModel tumorModel = new TumorModel();
    init.loadModel(tumorModel, null, false);
  }
}