package BostonTumor;

import uchicago.src.sim.engine.*; //import SimpleModel,SimInit,BasicAction
import uchicago.src.sim.analysis.DataRecorder; //Data Recorder: single value output
import java.util.*; //import Iterator, ArrayList
import uchicago.src.sim.analysis.DataFileHeader; //DataFileHeader: for Array output
import uchicago.src.sim.analysis.DataFileWriter; //DataFileWriter: for Array output
import java.io.*; //import PrintWriter

/**
 * Title:        Tumor Model 2004
 * Description:  Multiscale version (for Athale's Project)
 * Copyright:    Copyright (c) 2004
 * Company:      Massachusetts Institute of Technology
 * @author Yuri Mansury, Chaitanya Athale
 * @version 4.0
 */

public class TumorModelBatch
    extends TumorModel {

  DataRecorder recorder; // Repast's data recorder for cell level parameters
  DataRecorder recordnetw; // Repast's data recorder for network level parameters
//  DataRecorder coredgerecord; // Repast's data recorder for network level parameters
  //DataRecorder recordrna;// Rna expression of agent 5

//In order to access the entire array for a specific agent, we need DataFileWriter
  //different from DataRecorder
  DataFileHeader netHeader; // The header for the datafilewriter
  DataFileWriter netFile;   // The datafilewriter that outputs a string into a file

  // Initializing the model /////////////////////////////////////////////
  public void setup() {
    super.setup(); // Call the constructor of the super class
    System.gc(); // Call the garbage collector

    name = "Athale, Mansury & Deisboeck's Agent-Based Model of Brain Tumor Cells";

    // List inherited parameters that are of interest
    params = new String[] {
        "agentSize",
        //"rRho", // Intrinsic capability to move
        "rFood", // Rate of food depletion
        //"kProlif", // Controlling the proliferation prob.
        "TGFexi", // Initital TGF_ex comp 0
        "threshPLC", //The threshold for the PLCg decision to migr/prolif
        "threshPLCnoise", //threshold above noise of PLCg active conc.
       "threshpLRprol",//threshold of phos-TGFaEGFR to permit proliferation
       "errPLC",//the error of PLCgamma direction sensing
        "rLRint", //rate of EGFR and LR internalization
        "indTGFrna", //rate of TGF rna induction
        "initEGFRs" //initial conc of EGFR
             //"k1" // Rate of Synthesis of EGFR
    };
  }

  // Building up the internal objects of the model
  public void buildModel() {

    super.buildModel(); // Call the buildModel of the super class
    //setStoppingTime(runUntil) ;
    // Initialize the data recorder and the name of the output file:
    recorder = new DataRecorder("./TumorData.txt", this,
                                "Tumor-level Data output");
    //Output aspects of the state of the network
    recordnetw = new DataRecorder("./NetworkDataAg1.txt", this,
                                  "Network output-CA");
    //Output aspects of the state of the network
   //coredgerecord = new DataRecorder("./CoreEdgeProfile.txt", this,
     //                            "Spheroid Spatial MolecSign-CA");

    //recordrna = new DataRecorder("./NetworkDataAg1rna.txt", this,
      //                            "RNA Ag1 output-CA");

    // We set a "space" as the tabular data delimiter in the output file
    recordnetw.setDelimeter("   ");
    recordnetw.setDelimeter("   ");
    //coredgerecord.setDelimiter("   ");
    //recordrna.setDelimiter("   ");
    // Creates NumericDataSources using the specified method names
    // Get number of total agents
//    recorder.createNumericDataSource("nAgentTot", this, "getNAgentTot");
    // Get number of alive agents
    recorder.createNumericDataSource("nAgent", this, "getNAgent");

    // Get Number of migrating agents at a time pt
    recorder.createNumericDataSource("Migrating Cells", this,"getMigno");

//Get the number of proliferating cells at that time point
    recorder.createNumericDataSource("Proliferating Cells",this,"getProlno");

    // Get number of new daughter cells
//    recorder.createNumericDataSource("nNewAgent", this, "getNNewAgent");
    // Get time for the first agent to reach the third quadrant
    recorder.createNumericDataSource("tQuadrant3", this, "getTQuadrant3");

    // Get the Fractal dimension and R^2 value of the fit
    recorder.createNumericDataSource("fractalDim", this, "getfractalDim");

    // Get the Fractal dimension and R^2 value of the fit
    recorder.createNumericDataSource("Rsqval", this, "getRsqval");

    // Get the surface roughness Std Deviation
    recorder.createNumericDataSource("StdevDistEdge", this, "getStdevDist");

    // Get the Average surface radius
    recorder.createNumericDataSource("AverageEdgeDist", this, "getAvgdist");

    // Get the average inter-cellular disntace at the end of simulation in the
    // 3rd Quadrant
    recorder.createNumericDataSource("AvgmAgdist", this, "getAvgmAgdist");
    //migrating cell density in the 3rd quadrant
    recorder.createNumericDataSource("3qMigdensity", this, "get3qMigdensity");



//-------------------
//FOR THE NETWORK FILE

//Get the Total number of cells at that time point
    recordnetw.createNumericDataSource("Total Cells",this,"getNAgent");

    ///////For network recorder
    // Get the amt of autocrine secretion in a point in the grid
    recordnetw.createNumericDataSource("Autocrine", this, "getAutocrine");

   // Get the amt of external food already present in the same grid point
      recordnetw.createNumericDataSource("External Food", this, "getFoodext");

    // Get the PKC level in a given compartment of a single agent
    recordnetw.createNumericDataSource("Total PLC", this, "getAgent_PLC");

    // Get the pLR_s level in a given compartment of a single agent
    recordnetw.createNumericDataSource("Total pLR_s", this, "getAgpLR_s");

// Get Number of migrating agents at a time pt
    recordnetw.createNumericDataSource("migrating agents", this,"getMigno");

//Get the number of proliferating cells at that time point
    recordnetw.createNumericDataSource("Number Proliferating",this,"getProlno");

//Get the number of Edge cells at that time point
   recordnetw.createNumericDataSource("Edge Population",this,"getEdgeNumber");

//Get the number of Core cells at that time point
   recordnetw.createNumericDataSource("Core Population",this,"getCoreNumber");

   //Get the TGFrna concentration in the Core cells
   recordnetw.createNumericDataSource("Core TGFrna",this,"getcoreTGFrna");
   //Get the TGFrna concentration in the Edge cells
   recordnetw.createNumericDataSource("Edge TGFrna",this,"getedgeTGFrna");

   //Get the EGFRrna concentration in the Core cells
   recordnetw.createNumericDataSource("Core EGFRrna",this,"getcoreEGFRrna");
   //Get the EGFRrna concentration in the Edge cells
   recordnetw.createNumericDataSource("Edge EGFRrna",this,"getedgeEGFRrna");

   //Get the EGFR surface protein concentration in the Core cells
   recordnetw.createNumericDataSource("Core EGFRs",this,"getcoreEGFRs");
   //Get the EGFRs surface protein concentration in the Edge cells
   recordnetw.createNumericDataSource("Edge EGFRs",this,"getedgeEGFRs");
/////////////////////////
   //For final figure: prolif and mig molec average profiles
   //Get the Molecules for each spatial compartment
   recordnetw.createNumericDataSource("Mig TGFrna",this,"getmigTGFrna");
   recordnetw.createNumericDataSource("Mig EGFRrna",this,"getmigEGFRrna");
   recordnetw.createNumericDataSource("Mig PLC",this,"getmigPLC");

   recordnetw.createNumericDataSource("Pro EGFRrna",this,"getproTGFrna");
   recordnetw.createNumericDataSource("Pro EGFRrna",this,"getproEGFRrna");
   recordnetw.createNumericDataSource("Pro PLC",this,"getproPLC");
/*   recordnetw.createNumericDataSource("Mig EGFRrna",this,"getmigTGFaex");
   recordnetw.createNumericDataSource("Mig EGFRrna",this,"getmigPpLR");
   recordnetw.createNumericDataSource("Mig EGFRrna",this,"getmigPLCg");


   recordnetw.createNumericDataSource("Pro EGFRrna",this,"getproEGFRrna");
   recordnetw.createNumericDataSource("Pro EGFRrna",this,"getproTGFaex");
   recordnetw.createNumericDataSource("Pro EGFRrna",this,"getproPpLR");
   recordnetw.createNumericDataSource("Pro EGFRrna",this,"getproPLCg");

*/




// The data is written to the output file at the end of simulation:
    schedule.scheduleActionAtEnd(recorder, "writeToFile");
    schedule.scheduleActionAtEnd(recordnetw, "writeToFile");
   // schedule.scheduleActionAtEnd(coredgerecord, "writeToFile");
//Create a header and datafilewriter to output entire arrays
     netHeader = new DataFileHeader(this);
     netFile = new DataFileWriter("./Netfile.txt", netHeader );
  }

/////////////////////////////////////////////////////////////////////////
// Implementing Data Recorder Methods                                  //
/////////////////////////////////////////////////////////////////////////

  // The method that retrieves the number of total agents
//  public double getNAgentTot() {
//    return super.agentID;
//  }

  // The method that retrieves the number of alive agents
  public double getNAgent() {
    return super.getAgentList().size();
  }



//This method should return the number of agents moving
//public double getAgtot(){
 // return super.agentList.size();
//}


  // The method that retrieves the number of newly-generated daughter cells
//  public double getNNewAgent() {
//    return super.numNewAgent;
//  }

  // The method that retrieves the time to center of Quadrant III
  public double getTQuadrant3() {
    return super.timeToThirdQuadrant;
  }

  // The method that retrieves the fractal dimension
  public double getfractalDim() {
    return super.fitvals[0];
  }

  // The method that retrieves the R^2 value of the fitted fractal dimension
  public double getRsqval() {
    return super.fitvals[2];
  }

  public double getStdevDist() {
    return super.Roughness[0];
  }

  public double getAvgdist() {
    return super.Roughness[1];
  }

  public double getAvgmAgdist(){
    return super.avDistAg();
  }

  public double get3qMigdensity(){
    return (double)super.numMigAg3q;
  }

  // The method that retrieves the (actual) elapsed time (in seconds) to finish the simulation
  public double getElapsedTimeSecond() {
    return super.elapsedTimeSecond;
  }

  //The method that retrieves autocrine secretion in a particular locationID
  public double getAutocrine() {
    double autocrLevel;
    autocrLevel = super.autocrFood[20100];
    return autocrLevel;
  }

  //The method that retrieves autocrine secretion in a particular locationID
  public double getFoodext() {
    double extfdLevel;
    extfdLevel = super.foodLevel[20100];
    return extfdLevel;
  }

  //The method retrieves the specific value of PKC in compartment 2(0:3)
  //of an agent with agentID = 1
  public double getAgent_PLC() {
    double PLCLevel = 0.0;
    Iterator it1 = agentList.iterator();
    while (it1.hasNext()) {
      Agent a1 = (Agent) it1.next();
      if(a1.agentID==0){
        for(int compn=0;compn<4;compn++){
          PLCLevel += a1.Xmol_new[compn][9];//active PLC gamma in all 4 compartments
        }
           //System.out.println("Total PKCLevel Agent1 = "+ PLCLevel);
         }
    }
    return PLCLevel;
  }
  //The method retrieves the specific value of TGFaEGFR_s in compartment 2(0:3)
  //of an agent with agentID = 1
  public double getAgpLR_s() {
    double LR_asLevel = 0.0;
    Iterator it1 = agentList.iterator();
    while (it1.hasNext()) {
      Agent a1 = (Agent) it1.next();
      if(a1.agentID==0){
        for(int compn=0;compn<4;compn++){
          LR_asLevel += a1.Xmol_new[compn][3];
        }
           //System.out.println("tgfa-egfr_s Level A1 c3= "+  TGFaEGFR_sLevel);
         }
    }
    return LR_asLevel;
  }

//This method returns the number of agents moving
public double getMigno(){
    //return super.getMigList().size();
    return Agent.alreadyMoved;
    //return super.movingList.size();
    //return super.getmovingList().size();
  }

//This method returns the number of agents dividing
public double getProlno(){
    //return super.numNewAgent;//number of new "daughter" agents
    double totprolif = super.getProlifList().size();// + super.numNewAgent;
    return totprolif;
}

// This method returns the number of agents on the edge
public double getEdgeNumber(){
    return super.getEdgeList().size();//number of new "daughter" agents
}

// This method returns number of agents in the core
public double getCoreNumber(){
   return super.getCoreList().size();//number of new "daughter" agents
}

// Returns concetentration of TGFrna in core cells
 public double getcoreTGFrna() {
   double cTGFrna = 0;
   Iterator itc = coreList.iterator();
   while (itc.hasNext()) {
     Agent ac = (Agent) itc.next();
     for (int compn = 0; compn < 4; compn++) {
       cTGFrna += ac.Xmol_new[compn][7]; //EGFR rna
     }
   }
   if (coreList.size() > 0) {
     cTGFrna = cTGFrna / coreList.size();
   }
   //cTGFrna = coreTGFrna;
   return cTGFrna; //core EGFR rna
 }

// Returns concetentration of TGFrna in edge cells
public double getedgeTGFrna(){
   double eTGFrna = 0;
   Iterator ite = edgeList.iterator();
   while (ite.hasNext()) {
     Agent ae = (Agent) ite.next();
       for (int compn = 0; compn < 4; compn++) {
         eTGFrna += ae.Xmol_new[compn][7]; //EGFR rna
       }
       //System.out.println("Total PKCLevel Agent1 = "+ PLCLevel);
   }
   if (edgeList.size() > 0) {
    eTGFrna = eTGFrna / edgeList.size();
  }
return eTGFrna; //edge EGFR rna
}

// Returns concetentration of EGFRrna in core cells
public double getcoreEGFRrna() {
  double cEGFRrna = 0;
  Iterator itc = coreList.iterator();
  while (itc.hasNext()) {
    Agent ac = (Agent) itc.next();
    for (int compn = 0; compn < 4; compn++) {
      cEGFRrna += ac.Xmol_new[compn][8]; //EGFR rna
    }
    //System.out.println("Total PKCLevel Agent1 = "+ PLCLevel);
  }
  if (coreList.size() > 0) {
    cEGFRrna = cEGFRrna / coreList.size();
  }
  return cEGFRrna; //edge EGFR rna
}

//Returns concetentration of EGFRrna in edge cells
public double getedgeEGFRrna(){
  double eEGFRrna = 0;
  Iterator ite = edgeList.iterator();
  while (ite.hasNext()) {
    Agent ae = (Agent) ite.next();
    for (int compn = 0; compn < 4; compn++) {
      eEGFRrna += ae.Xmol_new[compn][8]; //EGFR rna
    }
    //System.out.println("Total PKCLevel Agent1 = "+ PLCLevel);
  }
  if (edgeList.size() > 0) {
    eEGFRrna = eEGFRrna / edgeList.size();
  }
   return eEGFRrna; //edge EGFR rna
}

//Returns concetentration of surfaceEGFR prot in core cells
public double getcoreEGFRs(){
   double cEGFRs = 0;
   Iterator itc = coreList.iterator();
   while (itc.hasNext()) {
     Agent ac = (Agent) itc.next();
     for (int compn = 0; compn < 4; compn++) {
       cEGFRs += ac.Xmol_new[compn][1] + //surface EGFR free
           +ac.Xmol_new[compn][2] //surface EGFR free
           + ac.Xmol_new[compn][3]; //surface EGFR free
     }
     //System.out.println("Total PKCLevel Agent1 = "+ PLCLevel);
   }
   if (coreList.size() > 0) {
     cEGFRs = cEGFRs / coreList.size();
   }

   return cEGFRs;//core number of surface EGFR
}

//Returns concetentration of surfaceEGFR prot in edge cells
 public double getedgeEGFRs() {
   double eEGFRs = 0;
   Iterator ite = edgeList.iterator();
   while (ite.hasNext()) {
     Agent ae = (Agent) ite.next();
     for (int compn = 0; compn < 4; compn++) {
       eEGFRs += ae.Xmol_new[compn][1] + //surface EGFR free
           +ae.Xmol_new[compn][2] //surface EGFR free
           + ae.Xmol_new[compn][3]; //surface EGFR free
     }
     //System.out.println("Total PKCLevel Agent1 = "+ PLCLevel);
   }
   if (edgeList.size() > 0) {
     eEGFRs = eEGFRs / edgeList.size();
   }
   //System.out.println("Total PKCLevel Agent1 = "+ PLCLevel);
 return eEGFRs; //total amt of EGFR surface molec at edge
}

//
 public double getmigTGFrna(){
   double migTGFrna = 0;
   Iterator itm = migList.iterator();
   while (itm.hasNext()) {
     Agent am = (Agent) itm.next();
     for (int compn = 0; compn < 4; compn++) {
       migTGFrna += am.Xmol_new[compn][7]; //mig TGF rna
     }
   }
   if (migList.size() > 0) {
     migTGFrna = migTGFrna / migList.size();
   }

 return migTGFrna;
 }

 public double getmigEGFRrna() {
   double migEGFRrna = 0;
   Iterator itm = migList.iterator();
   while (itm.hasNext()) {
     Agent am = (Agent) itm.next();
     for (int compn = 0; compn < 4; compn++) {
       migEGFRrna += am.Xmol_new[compn][8]; //mig TGF rna
     }
   }
   if (migList.size() > 0) {
     migEGFRrna = migEGFRrna / migList.size();
   }

   return migEGFRrna;
 }


 public double getmigPLC() {
   double migPLC = 0;
   Iterator itm = migList.iterator();
   while (itm.hasNext()) {
     Agent am = (Agent) itm.next();
     for (int compn = 0; compn < 4; compn++) {
       migPLC += am.Xmol_new[compn][9]; //mig TGF rna
     }
   }
   if (migList.size() > 0) {
     migPLC = migPLC / migList.size();
   }
   return migPLC;
 }



 public double getproTGFrna() {
   double proTGFrna = 0;
   Iterator itp = prolifList.iterator();
   while (itp.hasNext()) {
     Agent ap = (Agent) itp.next();
     for (int compn = 0; compn < 4; compn++) {
       proTGFrna += ap.Xmol_new[compn][7]; //mig TGF rna
     }
   }
   if (prolifList.size() > 0) {
     proTGFrna = proTGFrna / prolifList.size();
   }

   return proTGFrna;
 }

 public double getproEGFRrna(){
   double proEGFRrna = 0;
   Iterator itp = prolifList.iterator();
   while (itp.hasNext()) {
     Agent ap = (Agent) itp.next();
     for (int compn = 0; compn < 4; compn++) {
       proEGFRrna += ap.Xmol_new[compn][8]; //mig TGF rna
     }
   }
   if (prolifList.size() > 0) {
     proEGFRrna = proEGFRrna / prolifList.size();
   }

   return proEGFRrna;
  }

  public double getproPLC(){
    double proPLC = 0;
    Iterator itp = prolifList.iterator();
    while (itp.hasNext()) {
      Agent ap = (Agent) itp.next();
      for (int compn = 0; compn < 4; compn++) {
        proPLC += ap.Xmol_new[compn][9]; //mig TGF rna
      }
    }
    if (prolifList.size() > 0) {
      proPLC = proPLC / prolifList.size();
    }

    return proPLC;
   }

//


//The method that converts the array of a specific quantity (eg molecules)
  // into a string tab separated, for use with DataFileWriter
  public String toString(){
    String molstr = "Gen: PLCg: ";
    Iterator it2 = agentList.iterator();
    //Iterator it2 = prolifList.iterator();
    //Iterator it2 = migList.iterator();
    while (it2.hasNext()) {
      Agent a1 = (Agent) it2.next();
     if(a1.agentID==947){
       if(a1.isMig){
       molstr += "mig ";
     }
     if(a1.isProlif){
       molstr += "pro ";
     }
     if(a1.isQuiesc){
       molstr += "quies ";
     }

        for(int nn=0; nn<4; nn++){
          molstr += a1.Xmol_new[nn][9] + " , ";
        }
    }
  }
    return molstr;
  }

/////////////////////////////////////////////////////////////////////////
// RePast Parameter Panel Methods ///////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
// In the following we provide get/set methods for the parameters of interest
// we listed in params (see the setup() method)

  public int getAgentSize() {
    return agentSize;
  }

  public void setAgentSize(int n) {
    agentSize = n;
  }

/*  public double getRRho() {
    return rRho;
  }

  public void setRRho(double a) {
    rRho = a;
  }*/

  public double getRFood() {
    return rFood;
  }

  public void setRFood(double a) {
    rFood = a;
  }

/*  public double getKProlif() {
    return kProlif;
  }

  public void setKProlif(double a) {
    kProlif = a;
  }
  */
//get the initial TGFexternal conc COMPARTMENT 0
  public double getTGFexi(){
//     Iterator it = agentList.iterator();
//    while (it.hasNext()) {
//      Agent agent = (Agent) it.next();
//      for(int n = 0; n<4; n++){
//        super.TGFexi = agent.Xmol_ia[n][0];
//      }
//    }
    return TGFexi;
  }

  public void setTGFexi(double a) {
    TGFexi = a;
  }
// Thresholds
  public double getthreshPLC() {
     return threshPLC;
  }

  public void setthreshPLC(double a) {
    threshPLC = a;
  }

//
  public double getthreshPLCnoise() {
     return threshPLCnoise;
  }

  public void setthreshPLCnoise(double a) {
    threshPLCnoise = a;
  }
//
  public double getthreshpLRprol() {
     return threshpLRprol;
  }

  public void setthreshpLRprol(double a) {
    threshpLRprol = a;
  }

  public double geterrPLC() {
   return errPLC;
}

public void seterrPLC(double a) {
  errPLC = a;
}




  public double getrLRint() {
     return rLRint;
  }

  public void setrLRint(double a) {
    rLRint = a;
  }

  public double getindTGFrna() {
     return indTGFrna;
  }

  public void setindTGFrna(double a) {
    indTGFrna = a;
  }

  public double getinitEGFRs() {
      return initEGFRs;
   }

   public void setinitEGFRs(double a) {
     initEGFRs = a;
   }



//  public double getkf() {
//    return kf;
//  }

//  public void setkf(double a) {
//    kf = a;
//  }

///// Method carried out every time step ////////////////////////////////
  // Method of the class SimpleModel:
  public void atEnd() {
    // Get max. average cluster size
    // recorder.createNumericDataSource("maxAvgSize", this, "getMaxAvgSize") ;
    recorder.createNumericDataSource("elapsedTimeSecond", this,
                                     "getElapsedTimeSecond");
    recorder.record(); // Records the data for the current tick
    recorder.writeToFile(); // Write the data to output file

    //recordnetw.createNumericDataSource("elapsedTimeSecond", this, "getElapsedTimeSecond");
    //recordnetw.record(); // Records the data for the current tick
    //recordnetw.writeToFile(); // Write the data to output file
    //netFile.writeToFile(this);
  }

  // Method of the class SimpleModel:
  public void atPause() {
    atEnd();
  }

  // The postStep mehod
  public void postStep() {
    super.postStep(); // Call the postStep method of the super class
    netFile.writeToFile(this);
     recordnetw.record(); // Records the data for the current tick
     recordnetw.writeToFile(); // Write the data to output file
  }

/////////////////////////////////////////////////////////////////////////
// Creating and starting the model in batch mode ////////////////////////
/////////////////////////////////////////////////////////////////////////

  public static void main(String[] args) {
    SimInit init = new SimInit();
    TumorModelBatch t = new TumorModelBatch();
    init.loadModel(t, "./params.txt", true);
  }
}
