package BostonTumor;

import uchicago.src.sim.engine.*;   // import SimInit,Controller
import uchicago.src.sim.gui.* ; // import DisplayConstants,DisplaySurface

/**
 * Title:        Tumor Model Demo 2003
 * Description:  Demo version
 * Copyright:    Copyright (c) 2003
 * Company:      Massachusetts Institute of Technology & Mass. General Hospital (Harvard Medical School)
 * @author Yuri Mansury & Chaitanya Athale
 * @version 4.0
 */

public class TumorModelGUI extends TumorModel {

  // GUI variables
  public DisplaySurface dsurf ; // The display (grid) surface
  TumorDisplay tumorDisplay ; // The displayed object (i.e., the grid cell)
  boolean isSizeShown ; // On/off switch to display population values on tumorDisplay

  public TumorModelGUI() {
    Controller.ALPHA_ORDER = false;  // Non-alphabetic order of probed variables
  }

  // Initializing the model
  public void setup() {
    // The setup() method of the parent MUST be called first!
    super.setup() ;
    System.gc() ;        // Call the garbage collector

    name = "Athale, Mansury & Deisboeck's Agent + Signaling Net Model of Brain Tumor Cells" ;
    // Displays inherited parameters on a GUI panel
    params = new String[] {"resolution",
                           "agentSize",
                           "numMovers", // Max. number of moving agents
                           "isResistCrater", // Is resistance forming a crater
                           "isFoodPeak", // Is food peaking at quadrant 3
                           "rRho", // Intrinsic capability to move
                           "congest", // Congestion effect of overcrowding
                           "spill",  // The extent of spillover effect
                           "goodSigma", // Dispersion of nutrients
                           "resistanceSigma", // Dispersion of resistances
                           "qFood", // Coefficient of food in utility fcn.
                           //"qToxic", // Coefficient of toxicity in utility fcn.
                           "rFood",  // Rate of food depletion
                           "threshPLC", //Threshold parameter for PLCg active concentration
                           "threshPLCnoise", //noise threshold for PLCg active
                           "threshpLRprol", //noise threshold for PLCg active
                           "rResistance", // Rate of resistance decline
                           //"rToxic", // Rate of toxic accumulation
                           "gFood",  // Grow-back rate of food production
                           //"kProlif", // Controlling the proliferation prob.
                           //"kDeath", // Controlling the death probability
                           "isSizeShown",
                           //"isDeadCellShown", // Is dead cells shown
                           "isMigrCellShown" // Is migrating cells shown
                           } ;

    // If the display exists, let's dispose it!
    if (dsurf != null)
      dsurf.dispose();
    // Set the cell width & height in the display
    double width = (50.0/(double)resolution)*10.0 ;
    double height = (50.0/(double)resolution)*10.0 ;
    DisplayConstants.CELL_WIDTH = (int)Math.ceil(width) ;  // Static field
    DisplayConstants.CELL_HEIGHT = (int)Math.ceil(height) ; // Static field

    // Now it's time to create the display surface.
    dsurf = new DisplaySurface(this, "Multiscale Tumor Grid Display");
    // Registering it as part of our GUI
    registerDisplaySurface("Main", dsurf);

    isSizeShown = false ;
  }

  // Building up the GUI objects
  public void buildModel() {
    super.buildModel() ;
    // Create an instance of tumor display:
    tumorDisplay = new TumorDisplay(this) ;
    // We connect the tumor display with the display surface:
    dsurf.addDisplayable(tumorDisplay, "TumorDisplay") ;
    // Call a method of SimpleModel telling dsurf to listen to SimEvent
    // broadcast from this model
    this.addSimEventListener(dsurf) ;
    dsurf.display() ; // Display dsurf now (i.e., make it visible on screen)
  }

  // The pre-step method
  public void preStep() {
    super.preStep();
    dsurf.updateDisplay() ; // Update the on-screen display surface
  }

  // The post-step method
  public void postStep() {
    super.postStep() ;  // Call the post-step method of the super class first

    if (getTickCount() % 10 == 1)
      System.gc(); // Call the garbage collector
  }

/////////////////////////////////////////////////////////////////////////
// RePast Parameter Panel Methods ///////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
// In the following we provide get/set methods for all the parameters
// we listed in params (see the setup() method)
  public int getResolution() {return resolution ;}
  public void setResolution(int r) {resolution = r ;}
  public int getAgentSize() {return agentSize ;}
  public void setAgentSize(int a) {agentSize = a ;}
  public int getNumMovers() {return numMovers ;}
  public void setNumMovers(int a) {numMovers = a ;}
  public boolean getIsResistCrater() {return isResistCrater ;}
  public void setIsResistCrater(boolean a) {isResistCrater = a ;}
  public boolean getIsFoodPeak() {return isFoodPeak ;}
  public void setIsFoodPeak(boolean a) {isFoodPeak = a ;}
  public double getRRho() {return rRho ;}
  public void setRRho(double a) {rRho = a ;}
  public double getCongest() {return congest ;}
  public void setCongest(double a) {congest = a ;}
  public double getSpill() {return spill ;}
  public void setSpill(double a) {spill = a ;}
  public double getGoodSigma() {return goodSigma ;}
  public void setGoodSigma(double a) {goodSigma = a ;}
  public double getResistanceSigma() {return resistanceSigma ;}
  public void setResistanceSigma(double a) {resistanceSigma = a ;}
  public double getQFood() {return qFood ;}
  public void setQFood(double a) {qFood = a ;}
  //public double getQToxic() {return qToxic ;}
  //public void setQToxic(double a) {qToxic = a ;}
  public double getthreshPLC() {return threshPLC;} //Threshold parameter for PLCg active concentration
  public void setthreshPLC(double a) {threshPLC=a;}
  public double getthreshPLCnoise() {return threshPLCnoise;} //noise threshold for PLCg active
  public void setthreshPLCnoise(double a) {threshPLCnoise=a;}
  public double getthreshpLRprol() {return threshpLRprol;} //noise threshold for PLCg active
  public void setthreshpLRprol(double a) {threshpLRprol = a;}
  public double getRFood() {return rFood ;}
  public void setRFood(double a) {rFood = a ;}
  public double getRResistance() { return rResistance ; }
  public void setRResistance(double a) { rResistance = a ; }
  //public double getRToxic() {return rToxic ;}
  //public void setRToxic(double a) {rToxic = a ;}
  public double getGFood() {return gFood ;}
  public void setGFood(double a) {gFood = a ;}
  //public double getKProlif() {return kProlif ;}
  //public void setKProlif(double a) {kProlif = a ;}
  //public double getkDeath() {return kDeath ;}
  //public void setkDeath(double a) {kDeath = a ;}
  public boolean getIsSizeShown() {return isSizeShown ;}
  public void setIsSizeShown(boolean i) {isSizeShown = i ;}
//  public boolean getIsDeadCellShown() {return isDeadCellShown ;}
//  public void setIsDeadCellShown(boolean i) {isDeadCellShown = i ;}
  public boolean getIsMigrCellShown() {return isMigrCellShown ;}
  public void setIsMigrCellShown(boolean i) {isMigrCellShown = i ;}


/////////////////////////////////////////////////////////////////////////
// Creating and starting the tumor model  ///////////////////////////////
/////////////////////////////////////////////////////////////////////////
  public static void main(String[] args) {
    SimInit init = new SimInit();
    TumorModelGUI model = new TumorModelGUI();
    init.loadModel(model, "", false);
  }

}